@@ -1,20 +1,18 @@
-#define SP_DOCUMENT_C
+#define __SP_DOCUMENT_C__
 
 /*
- * SPDocument
- *
- * This is toplevel class, implementing a gateway from repr to most
- * editing properties, like canvases (desktops), undo stacks etc.
+ * SVG document implementation
  *
  * Authors:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2000 Lauris Kaplinski
- * Copyright (C) 2001-2002 Lauris Kaplinski and Ximian, Inc.
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
  *
- * Distributed under GNU General Public License
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define noSP_DOCUMENT_DEBUG_IDLE
 #define noSP_DOCUMENT_DEBUG_UNDO
 
 #include <config.h>
@@ -30,8 +28,12 @@
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 
-#define A4_WIDTH  (21.0 * 72.0 / 2.54)
-#define A4_HEIGHT (29.7 * 72.0 / 2.54)
+#define SP_NAMESPACE_SVG "http://www.w3.org/2000.svg"
+#define SP_NAMESPACE_SODIPODI "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
+#define SP_NAMESPACE_XLINK "http://www.w3.org/1999/xlink"
+
+#define A4_WIDTH_STR "210mm"
+#define A4_HEIGHT_STR "297mm"
 
 enum {
 	MODIFIED,
@@ -113,14 +115,12 @@ sp_document_init (SPDocument *doc)
 
 	doc->modified_id = 0;
 
-
+	doc->rdoc = NULL;
+	doc->rroot = NULL;
+	doc->root = NULL;
 
 	p = g_new (SPDocumentPrivate, 1);
 
-	p->rdoc = NULL;
-	p->rroot = NULL;
-
-	p->root = NULL;
 
 	p->iddef = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -168,11 +168,11 @@ sp_document_destroy (GtkObject *object)
 
 		if (private->uri) g_free (private->uri);
 
-		if (private->root) gtk_object_unref (GTK_OBJECT (private->root));
+		if (doc->root) gtk_object_unref (GTK_OBJECT (doc->root));
 
 		if (private->iddef) g_hash_table_destroy (private->iddef);
 
-		if (private->rdoc) sp_repr_document_unref (private->rdoc);
+		if (doc->rdoc) sp_repr_document_unref (doc->rdoc);
 
 		/* Free resources */
 		g_hash_table_foreach_remove (private->resources, sp_document_resource_list_free, doc);
@@ -191,13 +191,13 @@ sp_document_destroy (GtkObject *object)
 }
 
 SPDocument *
-sp_document_new (const gchar * uri)
+sp_document_new (const gchar *uri)
 {
-	SPDocument * document;
-	SPReprDoc * rdoc;
-	SPRepr * rroot;
-	SPObject * object;
-	gchar * b;
+	SPDocument *document;
+	SPReprDoc *rdoc;
+	SPRepr *rroot;
+	guint version;
+	gchar *b;
 	gint len;
 
 	if (uri != NULL) {
@@ -212,25 +212,14 @@ sp_document_new (const gchar * uri)
 	} else {
 		rdoc = sp_repr_document_new ("svg");
 		rroot = sp_repr_document_root (rdoc);
-		g_return_val_if_fail (rroot != NULL, NULL);
-		sp_repr_set_attr (rroot, "style",
-			"fill:#000000;fill-opacity:0.5;stroke:none");
+		sp_repr_set_attr (rroot, "style", "fill:#000000;stroke:none;");
 	}
-	/* A quick hack to get namespaces into doc */
-	sp_repr_set_attr (rroot, "xmlns", "http://www.w3.org/2000/svg");
-	sp_repr_set_attr (rroot, "xmlns:sodipodi", "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd");
-	sp_repr_set_attr (rroot, "xmlns:xlink", "http://www.w3.org/1999/xlink");
-	/* End of quick hack */
-	/* Quick hack 2 - get default image size into document */
-	if (!sp_repr_attr (rroot, "width")) sp_repr_set_double_attribute (rroot, "width", A4_WIDTH);
-	if (!sp_repr_attr (rroot, "height")) sp_repr_set_double_attribute (rroot, "height", A4_HEIGHT);
-	/* End of quick hack 2 */
 
 	document = gtk_type_new (SP_TYPE_DOCUMENT);
 	g_return_val_if_fail (document != NULL, NULL);
 
-	document->private->rdoc = rdoc;
-	document->private->rroot = rroot;
+	document->rdoc = rdoc;
+	document->rroot = rroot;
 
 	if (uri != NULL) {
 		b = g_strdup (uri);
@@ -242,35 +231,59 @@ sp_document_new (const gchar * uri)
 		document->private->base = g_malloc (len);
 		g_snprintf (document->private->base, len, "%s/", b);
 		g_free (b);
-		sp_repr_set_attr (rroot, "sodipodi:docname", uri);
-		sp_repr_set_attr (rroot, "sodipodi:docbase", document->private->base);
 	} else {
 		document->private->uri = g_strdup_printf (_("Unnamed document %d"), ++doc_count);
 	}
 
-	object = sp_object_repr_build_tree (document, rroot);
-	g_return_val_if_fail (object != NULL, NULL);
-	g_return_val_if_fail (SP_IS_ROOT (object), NULL);
+	document->root = sp_object_repr_build_tree (document, rroot);
+	g_return_val_if_fail (document->root != NULL, NULL);
+	g_return_val_if_fail (SP_IS_ROOT (document->root), NULL);
+	version = SP_ROOT (document->root)->sodipodi;
 
-	document->private->root = SP_ROOT (object);
+	/* fixme: Not sure about this, but lets assume ::build updates */
+	sp_repr_set_attr (rroot, "sodipodi:version", VERSION);
+	/* fixme: Again, I moved these here to allow version determining in ::build (Lauris) */
+	/* A quick hack to get namespaces into doc */
+	sp_repr_set_attr (rroot, "xmlns", SP_NAMESPACE_SVG);
+	sp_repr_set_attr (rroot, "xmlns:sodipodi", SP_NAMESPACE_SODIPODI);
+	sp_repr_set_attr (rroot, "xmlns:xlink", SP_NAMESPACE_XLINK);
+	/* End of quick hack */
+	/* Quick hack 2 - get default image size into document */
+	if (!sp_repr_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
+	if (!sp_repr_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
+	/* End of quick hack 2 */
+	/* Quick hack 3 - Set uri attributes */
+	if (uri) {
+		/* fixme: Think, what this means for images (Lauris) */
+		sp_repr_set_attr (rroot, "sodipodi:docname", uri);
+		sp_repr_set_attr (rroot, "sodipodi:docbase", document->private->base);
+	}
+	/* End of quick hack 3 */
+	if ((version > 0) && (version < 25)) {
+		/* Clear ancient spec violating attributes */
+		sp_repr_set_attr (rroot, "SP-DOCNAME", NULL);
+		sp_repr_set_attr (rroot, "SP-DOCBASE", NULL);
+		sp_repr_set_attr (rroot, "docname", NULL);
+		sp_repr_set_attr (rroot, "docbase", NULL);
+	}
 
 	/* Namedviews */
-	if (!document->private->root->namedviews) {
+	if (!SP_ROOT (document->root)->namedviews) {
 		SPRepr * r;
 		r = sp_repr_new ("sodipodi:namedview");
 		sp_repr_set_attr (r, "id", "base");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
-		g_assert (document->private->root->namedviews);
+		g_assert (SP_ROOT (document->root)->namedviews);
 	}
 
 	/* Defs */
-	if (!document->private->root->defs) {
+	if (!SP_ROOT (document->root)->defs) {
 		SPRepr *r;
 		r = sp_repr_new ("defs");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
-		g_assert (document->private->root->defs);
+		g_assert (SP_ROOT (document->root)->defs);
 	}
 
 	sodipodi_ref ();
@@ -283,12 +296,12 @@ sp_document_new (const gchar * uri)
 }
 
 SPDocument *
-sp_document_new_from_mem (const gchar * buffer, gint length)
+sp_document_new_from_mem (const gchar *buffer, gint length)
 {
-	SPDocument * document;
-	SPReprDoc * rdoc;
-	SPRepr * rroot;
-	SPObject * object;
+	SPDocument *document;
+	SPReprDoc *rdoc;
+	SPRepr *rroot;
+	guint version;
 
 	rdoc = sp_repr_read_mem (buffer, length);
 
@@ -301,49 +314,58 @@ sp_document_new_from_mem (const gchar * buffer, gint length)
 	/* fixme: destroy document */
 	if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
 
-	/* A quick hack to get namespaces into doc */
-	sp_repr_set_attr (rroot, "xmlns", "http://www.w3.org/2000.svg");
-	sp_repr_set_attr (rroot, "xmlns:sodipodi", "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd");
-	sp_repr_set_attr (rroot, "xmlns:xlink", "http://www.w3.org/1999/xlink");
-	/* End of quick hack */
-	/* Quick hack 2 - get default image size into document */
-	if (!sp_repr_attr (rroot, "width")) sp_repr_set_double_attribute (rroot, "width", A4_WIDTH);
-	if (!sp_repr_attr (rroot, "height")) sp_repr_set_double_attribute (rroot, "height", A4_HEIGHT);
-	/* End of quick hack 2 */
-
 	document = gtk_type_new (SP_TYPE_DOCUMENT);
 	g_return_val_if_fail (document != NULL, NULL);
 
-	document->private->rdoc = rdoc;
-	document->private->rroot = rroot;
+	document->rdoc = rdoc;
+	document->rroot = rroot;
 
 	document->private->uri = g_strdup_printf (_("Memory document %d"), ++doc_count);
 
-	object = sp_object_repr_build_tree (document, rroot);
-	g_return_val_if_fail (object != NULL, NULL);
-	g_return_val_if_fail (SP_IS_ROOT (object), NULL);
-
-	document->private->root = SP_ROOT (object);
+	document->root = sp_object_repr_build_tree (document, rroot);
+	g_return_val_if_fail (document->root != NULL, NULL);
+	g_return_val_if_fail (SP_IS_ROOT (document->root), NULL);
+	version = SP_ROOT (document->root)->sodipodi;
 
 	/* fixme: docbase */
 
+	/* fixme: Not sure about this, but lets assume ::build updates */
+	sp_repr_set_attr (rroot, "sodipodi:version", VERSION);
+	/* fixme: Again, I moved these here to allow version determining in ::build (Lauris) */
+	/* A quick hack to get namespaces into doc */
+	sp_repr_set_attr (rroot, "xmlns", SP_NAMESPACE_SVG);
+	sp_repr_set_attr (rroot, "xmlns:sodipodi", SP_NAMESPACE_SODIPODI);
+	sp_repr_set_attr (rroot, "xmlns:xlink", SP_NAMESPACE_XLINK);
+	/* End of quick hack */
+	/* Quick hack 2 - get default image size into document */
+	if (!sp_repr_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
+	if (!sp_repr_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
+	/* End of quick hack 2 */
+	if ((version > 0) && (version < 25)) {
+		/* Clear ancient spec violating attributes */
+		sp_repr_set_attr (rroot, "SP-DOCNAME", NULL);
+		sp_repr_set_attr (rroot, "SP-DOCBASE", NULL);
+		sp_repr_set_attr (rroot, "docname", NULL);
+		sp_repr_set_attr (rroot, "docbase", NULL);
+	}
+
 	/* Namedviews */
-	if (!document->private->root->namedviews) {
+	if (!SP_ROOT (document->root)->namedviews) {
 		SPRepr * r;
 		r = sp_repr_new ("sodipodi:namedview");
 		sp_repr_set_attr (r, "id", "base");
 		sp_repr_add_child (rroot, r, 0);
 		sp_repr_unref (r);
-		g_assert (document->private->root->namedviews);
+		g_assert (SP_ROOT (document->root)->namedviews);
 	}
 
 	/* Defs */
-	if (!document->private->root->defs) {
+	if (!SP_ROOT (document->root)->defs) {
 		SPRepr *r;
 		r = sp_repr_new ("defs");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
-		g_assert (document->private->root->defs);
+		g_assert (SP_ROOT (document->root)->defs);
 	}
 
 	sodipodi_ref ();
@@ -377,44 +399,15 @@ sp_document_unref (SPDocument *doc)
 	return NULL;
 }
 
-SPReprDoc *
-sp_document_repr_doc (SPDocument * document)
-{
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (document->private != NULL, NULL);
-
-	return document->private->rdoc;
-}
-
-SPRepr *
-sp_document_repr_root (SPDocument * document)
-{
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (document->private != NULL, NULL);
-
-	return document->private->rroot;
-}
-
-SPRoot *
-sp_document_root (SPDocument * document)
-{
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (document->private != NULL, NULL);
-
-	return document->private->root;
-}
-
 gdouble
 sp_document_width (SPDocument * document)
 {
 	g_return_val_if_fail (document != NULL, 0.0);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
 	g_return_val_if_fail (document->private != NULL, 0.0);
+	g_return_val_if_fail (document->root != NULL, 0.0);
 
-	return document->private->root->width;
+	return SP_ROOT (document->root)->width.computed / 1.25;
 }
 
 gdouble
@@ -423,8 +416,9 @@ sp_document_height (SPDocument * document)
 	g_return_val_if_fail (document != NULL, 0.0);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
 	g_return_val_if_fail (document->private != NULL, 0.0);
+	g_return_val_if_fail (document->root != NULL, 0.0);
 
-	return document->private->root->height;
+	return SP_ROOT (document->root)->height.computed / 1.25;
 }
 
 const gchar *
@@ -456,14 +450,14 @@ sp_document_set_uri (SPDocument *document, const guchar *uri)
 }
 
 void
-sp_document_set_size (SPDocument *doc, gdouble width, gdouble height)
+sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height)
 {
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (SP_IS_DOCUMENT (doc));
 	g_return_if_fail (width > 0.001);
 	g_return_if_fail (height > 0.001);
 
-	gtk_signal_emit (GTK_OBJECT (doc), signals [RESIZED], width, height);
+	gtk_signal_emit (GTK_OBJECT (doc), signals [RESIZED], width / 1.25, height / 1.25);
 }
 
 const gchar *
@@ -484,7 +478,7 @@ sp_document_namedview_list (SPDocument * document)
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 
-	return document->private->root->namedviews;
+	return SP_ROOT (document->root)->namedviews;
 }
 
 SPNamedView *
@@ -495,7 +489,7 @@ sp_document_namedview (SPDocument * document, const gchar * id)
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 
-	nvl = document->private->root->namedviews;
+	nvl = SP_ROOT (document->root)->namedviews;
 	g_assert (nvl != NULL);
 
 	if (id == NULL) return SP_NAMEDVIEW (nvl->data);
@@ -525,21 +519,43 @@ sp_document_undef_id (SPDocument * document, const gchar * id)
 }
 
 SPObject *
-sp_document_lookup_id (SPDocument * document, const gchar * id)
+sp_document_lookup_id (SPDocument *doc, const gchar *id)
 {
-	return g_hash_table_lookup (document->private->iddef, id);
+	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+
+	return g_hash_table_lookup (doc->private->iddef, id);
 }
 
 /* Object modification root handler */
 
 void
-sp_document_request_modified (SPDocument *document)
+sp_document_request_modified (SPDocument *doc)
 {
-	if (!document->modified_id) {
-		document->modified_id = gtk_idle_add (sp_document_idle_handler, document);
+	if (!doc->modified_id) {
+		doc->modified_id = gtk_idle_add (sp_document_idle_handler, doc);
 	}
 }
 
+gint
+sp_document_ensure_up_to_date (SPDocument *doc)
+{
+	if (doc->modified_id) {
+		/* Remove handler */
+		gtk_idle_remove (doc->modified_id);
+		doc->modified_id = 0;
+		/* Emit "modified" signal on objects */
+		sp_object_modified (doc->root, 0);
+		/* Emit our own "modified" signal */
+		gtk_signal_emit (GTK_OBJECT (doc), signals [MODIFIED],
+				 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gint
 sp_document_idle_handler (gpointer data)
 {
@@ -560,13 +576,24 @@ sp_document_idle_handler (gpointer data)
 	/* ------------------------- */
 #endif
 
+#ifdef SP_DOCUMENT_DEBUG_IDLE
+	g_print ("->\n");
+#endif
+
 	/* Emit "modified" signal on objects */
-	sp_object_modified (SP_OBJECT (doc->private->root), 0);
+	sp_object_modified (doc->root, 0);
+
+#ifdef SP_DOCUMENT_DEBUG_IDLE
+	g_print ("\n->");
+#endif
 
 	/* Emit our own "modified" signal */
 	gtk_signal_emit (GTK_OBJECT (doc), signals [MODIFIED],
 			 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
 
+#ifdef SP_DOCUMENT_DEBUG_IDLE
+	g_print (" S ->\n");
+#endif
 	return FALSE;
 }
 
@@ -582,9 +609,9 @@ sp_document_add_repr (SPDocument *document, SPRepr *repr)
 	type = sp_repr_type_lookup (repr);
 
 	if (gtk_type_is_a (type, SP_TYPE_ITEM)) {
-		sp_repr_append_child (document->private->rroot, repr);
+		sp_repr_append_child (document->rroot, repr);
 	} else if (gtk_type_is_a (type, SP_TYPE_OBJECT)) {
-		sp_repr_append_child (SP_OBJECT_REPR (document->private->root->defs), repr);
+		sp_repr_append_child (SP_OBJECT_REPR (SP_ROOT (document->root)->defs), repr);
 	}
 
 	return sp_document_lookup_id (document, sp_repr_attr (repr, "id"));
@@ -611,14 +638,14 @@ sp_document_items_in_box (SPDocument * document, ArtDRect * box)
 	g_return_val_if_fail (document->private != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
-	group = SP_GROUP (document->private->root);
+	group = SP_GROUP (document->root);
 
 	s = NULL;
 
 	for (o = group->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
-			sp_item_bbox (child, &b);
+			sp_item_bbox_desktop (child, &b);
 			if ((b.x0 > box->x0) && (b.x1 < box->x1) &&
 			    (b.y0 > box->y0) && (b.y1 < box->y1)) {
 				s = g_slist_append (s, child);
@@ -650,14 +677,14 @@ sp_document_partial_items_in_box (SPDocument * document, ArtDRect * box)
 	g_return_val_if_fail (document->private != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
-	group = SP_GROUP (document->private->root);
+	group = SP_GROUP (document->root);
 
 	s = NULL;
 
 	for (o = group->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
 			child = SP_ITEM (o);
-			sp_item_bbox (child, &b);
+			sp_item_bbox_desktop (child, &b);
 			if ((((b.x0 > box->x0) && (b.x0 < box->x1)) ||
 			     ((b.x1 > box->x0) && (b.x1 < box->x1)))
 			    &&
