@@ -1,4 +1,15 @@
-#define SP_EVENT_BROKER_C
+#define __SP_EVENT_BROKER_C__
+
+/*
+ * Event context stuff
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include "select-context.h"
 #include "node-context.h"
@@ -17,10 +28,10 @@
 #include "event-broker.h"
 
 void
-sp_event_context_set_select (GtkWidget * widget)
+sp_event_context_set_select (gpointer data)
 {
 	if (SP_ACTIVE_DESKTOP) {
-		sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_SELECT_CONTEXT);
+		sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_SELECT_CONTEXT, "tools.select");
 		/* fixme: This is really ugly hack. We should bind bind and unbind class methods */
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, TRUE);
 		sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
@@ -28,90 +39,101 @@ sp_event_context_set_select (GtkWidget * widget)
 }
 
 void
-sp_event_context_set_node_edit (GtkWidget * widget)
+sp_event_context_set_node_edit (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_NODE_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_NODE_CONTEXT, "tools.nodes");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, TRUE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_rect (GtkWidget * widget)
+sp_event_context_set_rect (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_RECT_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_RECT_CONTEXT, "tools.shapes.rect");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_arc (GtkWidget * widget)
+sp_event_context_set_arc (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_ARC_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_ARC_CONTEXT, "tools.shapes.arc");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_star (GtkWidget * widget)
+sp_event_context_set_star (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_STAR_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_STAR_CONTEXT, "tools.shapes.star");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_spiral (GtkWidget * widget)
+sp_event_context_set_spiral (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_SPIRAL_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_SPIRAL_CONTEXT, "tools.shapes.spiral");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_freehand (GtkWidget * widget)
+sp_event_context_set_freehand (gpointer data)
+{
+	if (SP_ACTIVE_DESKTOP) {
+		sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_PENCIL_CONTEXT, "tools.freehand.pencil");
+		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
+		sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
+	}
+}
+
+void
+sp_event_context_set_pen (gpointer data)
+{
+	if (SP_ACTIVE_DESKTOP) {
+		sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_PEN_CONTEXT, "tools.freehand.pen");
+		/* fixme: */
+		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
+		sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
+	}
+}
+
+void
+sp_event_context_set_dynahand (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_DRAW_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_DYNA_DRAW_CONTEXT, "tools.calligraphic");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_dynahand (GtkWidget * widget)
+sp_event_context_set_text (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_DYNA_DRAW_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_TEXT_CONTEXT, "tools.text");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
 }
 
 void
-sp_event_context_set_text (GtkWidget * widget)
+sp_event_context_set_zoom (gpointer data)
 {
   if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_TEXT_CONTEXT);
-		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
-    sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
-  }
-}
-
-void
-sp_event_context_set_zoom (GtkWidget * widget)
-{
-  if (SP_ACTIVE_DESKTOP) {
-    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_ZOOM_CONTEXT);
+    sp_desktop_set_event_context (SP_ACTIVE_DESKTOP, SP_TYPE_ZOOM_CONTEXT, "tools.zoom");
 		sp_desktop_activate_guides (SP_ACTIVE_DESKTOP, FALSE);
     sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP));
   }
