@@ -4,17 +4,19 @@
  * Various utility methods for gradients
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * Copyright (C) 2001-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
  *
- * Released under GNU GPL
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <string.h>
 #include <ctype.h>
 
 #include "xml/repr-private.h"
+#include "style.h"
 #include "document-private.h"
 #include "sp-root.h"
 #include "gradient-chemistry.h"
@@ -162,9 +164,12 @@ sp_gradient_ensure_private_normalized (SPGradient *gr, SPGradient *vector)
 	g_return_val_if_fail (vector->state == SP_GRADIENT_STATE_VECTOR, NULL);
 
 	/* If we are already normalized private, change href and return */
-	if (gr->state == SP_GRADIENT_STATE_PRIVATE) {
+	if ((gr->state == SP_GRADIENT_STATE_PRIVATE) && (SP_OBJECT_HREFCOUNT (gr) > 1)) {
+#if 0
+		/* fixme: Have to remove this, because duplicating object clones gradient also */
 		/* This assertion is dangerous - we have to be very-very strict about it */
 		g_assert (SP_OBJECT_HREFCOUNT (gr) == 1);
+#endif
 		if (gr->href != vector) {
 			/* href is not vector */
 			sp_gradient_repr_set_link (SP_OBJECT_REPR (gr), vector);
@@ -323,7 +328,7 @@ sp_item_force_fill_lineargradient_vector (SPItem *item, SPGradient *gr)
 	style = SP_OBJECT_STYLE (item);
 	g_assert (style != NULL);
 
-	if ((style->fill.type != SP_PAINT_TYPE_PAINTSERVER) || !SP_IS_LINEARGRADIENT (style->fill.server)) {
+	if ((style->fill.type != SP_PAINT_TYPE_PAINTSERVER) || !SP_IS_LINEARGRADIENT (SP_STYLE_FILL_SERVER (style))) {
 		SPGradient *pg;
 		/* Current fill style is not lineargradient, so construct everything */
 		pg = sp_gradient_get_private_normalized (SP_OBJECT_DOCUMENT (item), gr);
@@ -331,8 +336,8 @@ sp_item_force_fill_lineargradient_vector (SPItem *item, SPGradient *gr)
 	} else {
 		SPGradient *ig;
 		/* Current fill style is lineargradient */
-		ig = SP_GRADIENT (style->fill.server);
-		if (ig->state != SP_GRADIENT_STATE_PRIVATE) {
+		ig = SP_GRADIENT (SP_STYLE_FILL_SERVER (style));
+		if ((ig->state != SP_GRADIENT_STATE_PRIVATE) || (SP_OBJECT_HREFCOUNT (ig) != 1)) {
 			SPGradient *pg;
 			/* Check, whether we have to normalize private gradient */
 			pg = sp_gradient_ensure_private_normalized (ig, gr);
@@ -372,7 +377,7 @@ sp_item_force_stroke_lineargradient_vector (SPItem *item, SPGradient *gr)
 	style = SP_OBJECT_STYLE (item);
 	g_assert (style != NULL);
 
-	if ((style->stroke.type != SP_PAINT_TYPE_PAINTSERVER) || !SP_IS_LINEARGRADIENT (style->stroke.server)) {
+	if ((style->stroke.type != SP_PAINT_TYPE_PAINTSERVER) || !SP_IS_LINEARGRADIENT (SP_STYLE_STROKE_SERVER (style))) {
 		SPGradient *pg;
 		/* Current fill style is not lineargradient, so construct everything */
 		pg = sp_gradient_get_private_normalized (SP_OBJECT_DOCUMENT (item), gr);
@@ -380,8 +385,8 @@ sp_item_force_stroke_lineargradient_vector (SPItem *item, SPGradient *gr)
 	} else {
 		SPGradient *ig;
 		/* Current fill style is lineargradient */
-		ig = SP_GRADIENT (style->stroke.server);
-		if (ig->state != SP_GRADIENT_STATE_PRIVATE) {
+		ig = SP_GRADIENT (SP_STYLE_STROKE_SERVER (style));
+		if ((ig->state != SP_GRADIENT_STATE_PRIVATE) || (SP_OBJECT_HREFCOUNT (ig) != 1)) {
 			SPGradient *pg;
 			/* Check, whether we have to normalize private gradient */
 			pg = sp_gradient_ensure_private_normalized (ig, gr);
@@ -430,6 +435,7 @@ sp_gradient_repr_set_link (SPRepr *repr, SPGradient *link)
 static void
 sp_item_repr_set_style_gradient (SPRepr *repr, const guchar *property, SPGradient *gr)
 {
+#if 0
 	const guchar *sstr;
 	guchar *val, *newval;
 
@@ -443,6 +449,21 @@ sp_item_repr_set_style_gradient (SPRepr *repr, const guchar *property, SPGradien
 	g_free (val);
 	sp_repr_set_attr (repr, "style", newval);
 	g_free (newval);
+#else
+	SPCSSAttr *css;
+	guchar *val;
+
+	g_return_if_fail (repr != NULL);
+	g_return_if_fail (gr != NULL);
+	g_return_if_fail (SP_IS_GRADIENT (gr));
+
+	val = g_strdup_printf ("url(#%s)", SP_OBJECT_ID (gr));
+	css = sp_repr_css_attr_new ();
+	sp_repr_css_set_property (css, property, val);
+	g_free (val);
+	sp_repr_css_change_recursive (repr, css, "style");
+	sp_repr_css_attr_unref (css);
+#endif
 }
 
 /*
@@ -531,13 +552,13 @@ sp_object_ensure_fill_gradient_normalized (SPObject *object)
 	/* Need style */
 	g_return_if_fail (object->style != NULL);
 	/* Fill has to be set */
-	g_return_if_fail (object->style->fill_set);
+	g_return_if_fail (object->style->fill.set);
 	/* Fill has to be paintserver */
 	g_return_if_fail (object->style->fill.type = SP_PAINT_TYPE_PAINTSERVER);
 	/* Has to be linear gradient */
-	g_return_if_fail (SP_IS_LINEARGRADIENT (object->style->fill.server));
+	g_return_if_fail (SP_IS_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object)));
 
-	lg = SP_LINEARGRADIENT (object->style->fill.server);
+	lg = SP_LINEARGRADIENT (SP_OBJECT_STYLE_FILL_SERVER (object));
 
 	if (SP_OBJECT_HREFCOUNT (lg) > 1) {
 		const guchar *sstr;
