@@ -1,5 +1,19 @@
-#define CURVE_C
+#define __CURVE_C__
 
+/*
+ * Wrapper around ArtBpath
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ * Copyright (C) 2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL
+ */
+
+#include <math.h>
 #include <string.h>
 #include <libart_lgpl/art_misc.h>
 #include "curve.h"
@@ -7,6 +21,7 @@
 #define SP_CURVE_LENSTEP 32
 
 static gboolean sp_bpath_good (ArtBpath * bpath);
+static ArtBpath *sp_bpath_clean (ArtBpath *bpath);
 ArtBpath * sp_bpath_check_subpath (ArtBpath * bpath);
 static gint sp_bpath_length (ArtBpath * bpath);
 static gboolean sp_bpath_closed (ArtBpath * bpath);
@@ -34,9 +49,10 @@ sp_curve_new_sized (gint length)
 
 	curve->refcount = 1;
 	curve->bpath = art_new (ArtBpath, length);
+	curve->bpath->code = ART_END;
 	curve->end = 0;
-	curve->bpath[curve->end].code = ART_END;
 	curve->length = length;
+	curve->substart = 0;
 	curve->sbpath = FALSE;
 	curve->hascpt = FALSE;
 	curve->posset = FALSE;
@@ -47,11 +63,19 @@ sp_curve_new_sized (gint length)
 }
 
 SPCurve *
-sp_curve_new_from_bpath (ArtBpath * bpath)
+sp_curve_new_from_bpath (ArtBpath *bpath)
 {
-	SPCurve * curve;
+	SPCurve *curve;
 
-	g_return_val_if_fail (sp_bpath_good (bpath), NULL);
+	g_return_val_if_fail (bpath != NULL, NULL);
+
+	if (!sp_bpath_good (bpath)) {
+		ArtBpath *new;
+		new = sp_bpath_clean (bpath);
+		g_return_val_if_fail (new != NULL, NULL);
+		art_free (bpath);
+		bpath = new;
+	}
 
 	curve = g_new (SPCurve, 1);
 
@@ -71,9 +95,21 @@ sp_curve_new_from_bpath (ArtBpath * bpath)
 SPCurve *
 sp_curve_new_from_static_bpath (ArtBpath * bpath)
 {
-	SPCurve * curve;
+	SPCurve *curve;
+	gboolean sbpath;
+	gint i;
 
-	g_return_val_if_fail (sp_bpath_good (bpath), NULL);
+	g_return_val_if_fail (bpath != NULL, NULL);
+
+	if (!sp_bpath_good (bpath)) {
+		ArtBpath *new;
+		new = sp_bpath_clean (bpath);
+		g_return_val_if_fail (new != NULL, NULL);
+		sbpath = FALSE;
+		bpath = new;
+	} else {
+		sbpath = TRUE;
+	}
 
 	curve = g_new (SPCurve, 1);
 
@@ -81,7 +117,9 @@ sp_curve_new_from_static_bpath (ArtBpath * bpath)
 	curve->bpath = bpath;
 	curve->length = sp_bpath_length (bpath);
 	curve->end = curve->length - 1;
-	curve->sbpath = TRUE;
+	for (i = curve->end; i > 0; i--) if ((curve->bpath[i].code == ART_MOVETO) || (curve->bpath[i].code == ART_MOVETO_OPEN)) break;
+	curve->substart = i;
+	curve->sbpath = sbpath;
 	curve->hascpt = FALSE;
 	curve->posset = FALSE;
 	curve->moving = FALSE;
@@ -93,17 +131,24 @@ sp_curve_new_from_static_bpath (ArtBpath * bpath)
 SPCurve *
 sp_curve_new_from_foreign_bpath (ArtBpath * bpath)
 {
-	SPCurve * curve;
-	gint length;
+	SPCurve *curve;
+	ArtBpath *new;
 
-	g_return_val_if_fail (sp_bpath_good (bpath), NULL);
+	g_return_val_if_fail (bpath != NULL, NULL);
 
-	length = sp_bpath_length (bpath);
+	if (!sp_bpath_good (bpath)) {
+		new = sp_bpath_clean (bpath);
+		g_return_val_if_fail (new != NULL, NULL);
+	} else {
+		gint len;
+		len = sp_bpath_length (bpath);
+		new = art_new (ArtBpath, len);
+		memcpy (new, bpath, len * sizeof (ArtBpath));
+	}
 
-	curve = sp_curve_new_sized (length);
-	memcpy (curve->bpath, bpath, sizeof (ArtBpath) * length);
-	curve->end = length - 1;
-	curve->closed = sp_bpath_closed (bpath);
+	curve = sp_curve_new_from_bpath (new);
+
+	if (!curve) art_free (new);
 
 	return curve;
 }
@@ -193,29 +238,32 @@ sp_curve_concat (const GSList * list)
 	ArtBpath * bp;
 	const GSList * l;
 	gint length;
+	gint i;
 
 	g_return_val_if_fail (list != NULL, NULL);
 
-	length = 1;
+	length = 0;
 
 	for (l = list; l != NULL; l = l->next) {
 		c = (SPCurve *) l->data;
 		length += c->end;
 	}
 
-	new = sp_curve_new_sized (length);
+	new = sp_curve_new_sized (length + 1);
 
 	bp = new->bpath;
 
 	for (l = list; l != NULL; l = l->next) {
 		c = (SPCurve *) l->data;
-		memcpy (bp, c->bpath, c->end);
+		memcpy (bp, c->bpath, c->end * sizeof (ArtBpath));
 		bp += c->end;
 	}
 
 	bp->code = ART_END;
 
-	new->end = length - 1;
+	new->end = length;
+	for (i = new->end; i > 0; i--) if ((new->bpath[i].code == ART_MOVETO) || (new->bpath[i].code == ART_MOVETO_OPEN)) break;
+	new->substart = i;
 
 	return new;
 }
@@ -239,7 +287,9 @@ sp_curve_split (SPCurve * curve)
 		memcpy (new->bpath, curve->bpath + p, i * sizeof (ArtBpath));
 		new->end = i;
 		new->bpath[i].code = ART_END;
+		new->substart = 0;
 		new->closed = (new->bpath->code == ART_MOVETO);
+		new->hascpt = (new->bpath->code == ART_MOVETO_OPEN);
 		l = g_slist_append (l, new);
 		p += i;
 	}
@@ -247,6 +297,43 @@ sp_curve_split (SPCurve * curve)
 	return l;
 }
 
+SPCurve *
+sp_curve_transform (SPCurve *curve, const gdouble t[])
+{
+	gint i;
+
+	g_return_val_if_fail (curve != NULL, NULL);
+	g_return_val_if_fail (!curve->sbpath, NULL);
+	g_return_val_if_fail (t != NULL, curve);
+
+	for (i = 0; i < curve->end; i++) {
+		ArtBpath *p;
+		p = curve->bpath + i;
+		switch (p->code) {
+		case ART_MOVETO:
+		case ART_MOVETO_OPEN:
+		case ART_LINETO:
+			p->x3 = t[0] * p->x3 + t[2] * p->y3 + t[4];
+			p->y3 = t[1] * p->x3 + t[3] * p->y3 + t[5];
+			break;
+		case ART_CURVETO:
+			p->x1 = t[0] * p->x1 + t[2] * p->y1 + t[4];
+			p->y1 = t[1] * p->x1 + t[3] * p->y1 + t[5];
+			p->x2 = t[0] * p->x2 + t[2] * p->y2 + t[4];
+			p->y2 = t[1] * p->x2 + t[3] * p->y2 + t[5];
+			p->x3 = t[0] * p->x3 + t[2] * p->y3 + t[4];
+			p->y3 = t[1] * p->x3 + t[3] * p->y3 + t[5];
+			break;
+		default:
+			g_warning ("Illegal pathcode %d", p->code);
+			return NULL;
+			break;
+		}
+	}
+
+	return curve;
+}
+
 /* Methods */
 
 void
@@ -257,6 +344,7 @@ sp_curve_reset (SPCurve * curve)
 
 	curve->bpath->code = ART_END;
 	curve->end = 0;
+	curve->substart = 0;
 	curve->hascpt = FALSE;
 	curve->posset = FALSE;
 	curve->moving = FALSE;
@@ -582,27 +670,48 @@ sp_curve_append (SPCurve *curve,
                  gboolean use_lineto)
 {
 	ArtBpath *bs, *bp;
+	gboolean closed;
 
 	g_return_if_fail (curve != NULL);
 	g_return_if_fail (curve2 != NULL);
 
 	if (curve2->end < 1) return;
 
-	bs = curve2->bpath + curve2->substart;
+	bs = curve2->bpath;
 
+#if 0
 	if (use_lineto && (curve->end > 0)) {
 		sp_curve_lineto (curve, bs->x3, bs->y3);
+		closed = FALSE;
 	} else {
 		sp_curve_moveto (curve, bs->x3, bs->y3);
+		closed = FALSE;
 	}
+#endif
 
-	for (bp = bs+1; bp->code != ART_END; bp++) {
+	closed = curve->closed;
+
+	for (bp = bs; bp->code != ART_END; bp++) {
 		switch (bp->code) {
 		case ART_MOVETO_OPEN:
-			sp_curve_moveto (curve, bp->x3, bp->y3);
+			if (use_lineto && curve->hascpt) {
+				sp_curve_lineto (curve, bp->x3, bp->y3);
+				use_lineto = FALSE;
+			} else {
+				if (closed) sp_curve_closepath (curve);
+				sp_curve_moveto (curve, bp->x3, bp->y3);
+			}
+			closed = FALSE;
 			break;
 		case ART_MOVETO:
-			sp_curve_moveto (curve, bp->x3, bp->y3);
+			if (use_lineto && curve->hascpt) {
+				sp_curve_lineto (curve, bp->x3, bp->y3);
+				use_lineto = FALSE;
+			} else {
+				if (closed) sp_curve_closepath (curve);
+				sp_curve_moveto (curve, bp->x3, bp->y3);
+			}
+			closed = TRUE;
 			break;
 		case ART_LINETO:
 			sp_curve_lineto (curve, bp->x3, bp->y3);
@@ -614,12 +723,89 @@ sp_curve_append (SPCurve *curve,
 			g_assert_not_reached ();
 		}
 	}
+
+	if (closed) sp_curve_closepath (curve);
+}
+
+SPCurve *
+sp_curve_append_continuous (SPCurve *c0, SPCurve *c1, gdouble tolerance)
+{
+	ArtBpath *e;
+
+	g_return_val_if_fail (c0 != NULL, NULL);
+	g_return_val_if_fail (c1 != NULL, NULL);
+	g_return_val_if_fail (!c0->closed, NULL);
+	g_return_val_if_fail (!c1->closed, NULL);
+
+	if (c1->end < 1) return c0;
+
+	e = sp_curve_last_bpath (c0);
+	if (e) {
+		ArtBpath *s;
+		s = sp_curve_first_bpath (c1);
+		if (s && (fabs (s->x3 - e->x3) <= tolerance) && (fabs (s->y3 - e->y3) <= tolerance)) {
+			gboolean closed;
+			/* fixme: Strictly we mess in case of multisegment mixed open/close curves */
+			closed = FALSE;
+			for (s = s + 1; s->code != ART_END; s++) {
+				switch (s->code) {
+				case ART_MOVETO_OPEN:
+					if (closed) sp_curve_closepath (c0);
+					sp_curve_moveto (c0, s->x3, s->y3);
+					closed = FALSE;
+					break;
+				case ART_MOVETO:
+					if (closed) sp_curve_closepath (c0);
+					sp_curve_moveto (c0, s->x3, s->y3);
+					closed = TRUE;
+					break;
+				case ART_LINETO:
+					sp_curve_lineto (c0, s->x3, s->y3);
+					break;
+				case ART_CURVETO:
+					sp_curve_curveto (c0, s->x1, s->y1, s->x2, s->y2, s->x3, s->y3);
+					break;
+				case ART_END:
+					g_assert_not_reached ();
+				}
+			}
+		} else {
+			sp_curve_append (c0, c1, TRUE);
+		}
+	} else {
+		sp_curve_append (c0, c1, TRUE);
+	}
+
+	return c0;
+}
+
+void
+sp_curve_backspace (SPCurve *curve)
+{
+	g_return_if_fail (curve != NULL);
+
+	if (curve->end > 0) {
+		curve->end -= 1;
+		if (curve->end > 0) {
+			ArtBpath *p;
+			p = curve->bpath + curve->end - 1;
+			if ((p->code == ART_MOVETO) || (p->code == ART_MOVETO_OPEN)) {
+				curve->hascpt = TRUE;
+				curve->posset = TRUE;
+				curve->closed = FALSE;
+				curve->x = p->x3;
+				curve->y = p->y3;
+				curve->end -= 1;
+			}
+		}
+		curve->bpath[curve->end].code = ART_END;
+	}
 }
 
 /* Private methods */
 
-static
-gboolean sp_bpath_good (ArtBpath * bpath)
+static gboolean
+sp_bpath_good (ArtBpath *bpath)
 {
 	ArtBpath * bp;
 
@@ -637,6 +823,43 @@ gboolean sp_bpath_good (ArtBpath * bpath)
 	return TRUE;
 }
 
+static ArtBpath *
+sp_bpath_clean (ArtBpath *bpath)
+{
+	ArtBpath *new, *bp, *np;
+	gint len;
+
+	len = 0;
+	while (bpath[len].code != ART_END) len += 1;
+
+	new = art_new (ArtBpath, len + 1);
+
+	bp = bpath;
+	np = new;
+
+	while (bp->code != ART_END) {
+		if (sp_bpath_check_subpath (bp)) {
+			*np++ = *bp++;
+			while ((bp->code == ART_LINETO) || (bp->code == ART_CURVETO)) *np++ = *bp++;
+		} else {
+			*bp++;
+			while ((bp->code == ART_LINETO) || (bp->code == ART_CURVETO)) *bp++;
+		}
+	}
+
+	if (np == new) {
+		g_free (new);
+		return NULL;
+	}
+
+	np->code = ART_END;
+	np += 1;
+
+	new = art_renew (new, ArtBpath, np - new);
+
+	return new;
+}
+
 ArtBpath *
 sp_bpath_check_subpath (ArtBpath * bpath)
 {
