@@ -1,10 +1,15 @@
-#ifndef SODIPODI_CTRLLINE_H
-#define SODIPODI_CTRLLINE_H
+#ifndef __SODIPODI_CTRLLINE_H__
+#define __SODIPODI_CTRLLINE_H__
 
-/* sodipodi-ctrlline
- *
+/*
  * Simple straight line
  *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL
  */
 
 #include <libgnomeui/gnome-canvas.h>
@@ -17,13 +22,13 @@ BEGIN_GNOME_DECLS
 #define SP_IS_CTRLLINE(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_CTRLLINE))
 #define SP_IS_CTRLLINE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_CTRLLINE))
 
-
 typedef struct _SPCtrlLine SPCtrlLine;
 typedef struct _SPCtrlLineClass SPCtrlLineClass;
 
 struct _SPCtrlLine {
 	GnomeCanvasItem item;
 
+	guint32 rgba;
 	ArtPoint s, e;
 	ArtSVP * svp;
 };
@@ -32,11 +37,10 @@ struct _SPCtrlLineClass {
 	GnomeCanvasItemClass parent_class;
 };
 
-
-/* Standard Gtk function */
 GtkType sp_ctrlline_get_type (void);
 
-void sp_ctrlline_set_coords (SPCtrlLine * ctrlline, double x0, double y0, double x2, double y1);
+void sp_ctrlline_set_rgba32 (SPCtrlLine *cl, guint32 rgba);
+void sp_ctrlline_set_coords (SPCtrlLine *cl, gdouble x0, gdouble y0, gdouble x2, gdouble y1);
 
 END_GNOME_DECLS
 
