@@ -1,9 +1,25 @@
-#define SP_INTERFACE_C
+#define __SP_INTERFACE_C__
+
+/*
+ * Main UI stuff
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Frank Felfe <innerspace@iname.com>
+ *
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
+#include <config.h>
 
 #include <string.h>
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-mime.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-window-icon.h>
@@ -11,6 +27,7 @@
 #include "document.h"
 #include "desktop-handles.h"
 #include "file.h"
+#include "help.h"
 #include "interface.h"
 #include "desktop.h"
 #include "selection-chemistry.h"
@@ -31,6 +48,8 @@
 #include "dialogs/desktop-properties.h"
 #include "dialogs/document-properties.h"
 #include "dialogs/display-settings.h"
+#include "dialogs/tool-options.h"
+#include "dialogs/tool-attributes.h"
 
 void fake_dialogs (void);
 
@@ -47,6 +66,7 @@ static GtkTargetEntry ui_drop_target_entries [] = {
 static guint nui_drop_target_entries = ENTRIES_SIZE(ui_drop_target_entries);
 static void sp_ui_import_files(gchar * buffer);
 static void sp_ui_import_one_file(gchar * filename);
+static void sp_ui_import_one_file_with_check(gpointer filename, gpointer unused);
 static void sp_ui_drag_data_received (GtkWidget * widget,
 				      GdkDragContext * drag_context,
 				      gint x, gint y,
@@ -176,108 +196,37 @@ sp_ui_menu_append_item (GtkMenu *menu, const guchar *stock, const guchar *label,
 }
 
 static void
-sp_ui_file_menu (GtkMenu *menu, SPDocument *doc)
+sp_ui_file_menu (GtkMenu *fm, SPDocument *doc)
 {
-	GtkWidget *i;
-
-	/* File:New */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_NEW, _("New"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_new), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Open */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_OPEN, _("Open"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_open), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Save */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_SAVE, _("Save"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_save), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Save As */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_SAVE_AS, _("Save As"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_save_as), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Separator */
-	i = gtk_menu_item_new ();
-	gtk_widget_show (i);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Import */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_BLANK, _("Import"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_import), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Export */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_BLANK, _("Export"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_export), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Separator */
-	i = gtk_menu_item_new ();
-	gtk_widget_show (i);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Print */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_PRINT, _("Print"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_print), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Print Preview */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_BLANK, _("Print preview"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_print_preview), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Separator */
-	i = gtk_menu_item_new ();
-	gtk_widget_show (i);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Close */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_CLOSE, _("Close view"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_ui_close_view), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Exit */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_EXIT, _("Exit"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_file_exit), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_NEW, _("New"), sp_file_new, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_SAVE, _("Save"), sp_file_save, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_SAVE_AS, _("Save as"), sp_file_save_as, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Import"), sp_file_import, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Export"), sp_export_dialog, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_PRINT, _("Print"), sp_file_print, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_PRINT, _("Print Preview"), sp_file_print_preview, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_ABOUT, _("About Sodipodi"), sp_help_about, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_CLOSE, _("Close View"), sp_ui_close_view, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_EXIT, _("Exit Program"), sp_file_exit, NULL);
 }
 
 static void
-sp_ui_edit_menu (GtkMenu *menu, SPDocument *doc)
+sp_ui_edit_menu (GtkMenu *fm, SPDocument *doc)
 {
-	GtkWidget *i;
-
-	/* Edit:Cut */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_CUT, _("Cut"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_selection_cut), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Edit:Copy */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_COPY, _("Copy"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_selection_copy), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Edit:Paste */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_PASTE, _("Paste"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_selection_paste), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Separator */
-	i = gtk_menu_item_new ();
-	gtk_widget_show (i);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* Edit:Duplicate */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_BLANK, _("Duplicate"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_selection_duplicate), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
-	/* File:Export */
-	i = gnome_stock_menu_item (GNOME_STOCK_MENU_BLANK, _("Delete"));
-	gtk_widget_show (i);
-	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_selection_delete), NULL);
-	gtk_menu_append (GTK_MENU (menu), i);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_CUT, _("Cut"), sp_selection_cut, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_COPY, _("Copy"), sp_selection_copy, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_PASTE, _("Paste"), sp_selection_paste, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Duplicate"), sp_selection_duplicate, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Delete"), sp_selection_delete, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Clear all"), sp_edit_clear_all, NULL);
+	sp_ui_menu_append_item (GTK_MENU (fm), GNOME_STOCK_MENU_BLANK, _("Cleanup"), sp_edit_cleanup, NULL);
 }
 
 static void
@@ -388,16 +337,32 @@ sp_ui_dialog_menu (GtkMenu *menu, SPDocument *doc)
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Align Objects"), sp_quick_align_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Text Editing"), sp_text_edit_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Transformations"), sp_transformation_dialog, NULL);
+	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Tool Options"), sp_tool_options_dialog, NULL);
+	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Tool Attributes"), sp_tool_attributes_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Document"), sp_document_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Editing Window"), sp_desktop_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("XML Editor"), sp_xml_tree_dialog, NULL);
 	sp_ui_menu_append_item (menu, GNOME_STOCK_MENU_BLANK, _("Display Properties"), sp_display_dialog, NULL);
 }
 
-/*
- * Item properties
- *
- */
+/* Menus */
+
+GtkWidget *
+sp_ui_main_menu (void)
+{
+	GtkWidget *m;
+
+	m = gtk_menu_new ();
+
+	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_NEW, _("New"), sp_file_new, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_OPEN, _("Open"), sp_file_open, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_ABOUT, _("About Sodipodi"), sp_help_about, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
+	sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_EXIT, _("Exit Program"), sp_file_exit, NULL);
+
+	return m;
+}
 
 GtkWidget *
 sp_ui_generic_menu (SPView *v, SPItem *item)
@@ -437,37 +402,37 @@ sp_ui_generic_menu (SPView *v, SPItem *item)
 	/* File submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("File"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_file_menu (sm, NULL);
+	sp_ui_file_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 	/* Edit submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("Edit"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_edit_menu (sm, NULL);
+	sp_ui_edit_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 	/* Selection submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("Selection"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_selection_menu (sm, NULL);
+	sp_ui_selection_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 	/* View submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("View"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_view_menu (sm, NULL);
+	sp_ui_view_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 	/* Drawing mode submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("Drawing Mode"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_event_context_menu (sm, NULL);
+	sp_ui_event_context_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 	/* Dialog submenu */
 	i = sp_ui_menu_append_item (GTK_MENU (m), GNOME_STOCK_MENU_BLANK, _("Dialogs"), NULL, NULL);
 	sm = gtk_menu_new ();
-	sp_ui_dialog_menu (sm, NULL);
+	sp_ui_dialog_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
 
@@ -510,45 +475,21 @@ sp_ui_drag_data_received (GtkWidget * widget,
 	}
 }
 
-/* Most code of this function is copied from 
-   gimp-1.2.1/gimpdnd.c:gimp_dnd_file_open_files */
 static void
 sp_ui_import_files(gchar * buffer)
 {
-	gchar  name_buffer[1024];
-	const gchar *data_type 	   = "file:";
-	const gint   sig_len 	   = strlen (data_type);
-	gint  name_len;
+	GList * list = gnome_uri_list_extract_filenames(buffer);
+	if (!list)
+		return;
+	g_list_foreach (list, sp_ui_import_one_file_with_check, NULL);
+}
 
-	while (*buffer) {
-		gchar *name = name_buffer;
-		gint len = 0;
-
-		while ((*buffer != 0) && (*buffer != '\n') && len < 1024) {
-			*name++ = *buffer++;
-			len++;
-		}
-		if (len == 0)
-			break;
-
-		if (*(name - 1) == 0xd)   /* gmc uses RETURN+NEWLINE as delimiter */
-			*(name - 1) = '\0';
-		else
-			*name = '\0';
-		name = name_buffer;
-		if ((sig_len < len) && (! strncmp (name, data_type, sig_len)))
-			name += sig_len;
-			
-		name_len = name? strlen(name): 0;
-
-		/* SVG file */
-		if (name_len >  2) {
-		  sp_ui_import_one_file(name);
-		}
-		/* TODO: other file supports here... */
-		
-		if (*buffer)
-			buffer++;
+static void
+sp_ui_import_one_file_with_check(gpointer filename, gpointer unused)
+{
+	if (filename) {
+		if (strlen(filename) > 2)
+			sp_ui_import_one_file(filename);
 	}
 }
 
