@@ -1,13 +1,19 @@
-#define SP_KNOT_C
+#define __SP_KNOT_C__
 
 /*
- * SPKnot
+ * Desktop-bound visual control object
  *
- * Copyright (C) Lauris Kaplinski <lauris@ariman.ee> 2000
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <math.h>
+#include <gtk/gtksignal.h>
 #include "helper/sodipodi-ctrl.h"
 #include "desktop.h"
 #include "desktop-handles.h"
@@ -150,7 +156,7 @@ sp_knot_class_init (SPKnotClass * klass)
 		object_class->type,
 		GTK_SIGNAL_OFFSET (SPKnotClass, distance),
 		sp_marshal_DOUBLE__POINTER_UINT,
-		GTK_TYPE_DOUBLE, 3, GTK_TYPE_POINTER, GTK_TYPE_UINT);
+		GTK_TYPE_DOUBLE, 2, GTK_TYPE_POINTER, GTK_TYPE_UINT);
 
 	gtk_object_class_add_signals (object_class, knot_signals, LAST_SIGNAL);
 
