@@ -1,8 +1,23 @@
-#define SP_PATH_CHEMISTRY_C
+#define __SP_PATH_CHEMISTRY_C__
 
+/*
+ * Here are handlers for modifying selections, specific to paths
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
+#include <string.h>
 #include "xml/repr.h"
 #include "svg/svg.h"
 #include "sp-path.h"
+#include "sp-text.h"
+#include "style.h"
 #include "sodipodi.h"
 #include "document.h"
 #include "selection.h"
@@ -136,53 +151,100 @@ sp_selected_path_break_apart (void)
 void
 sp_selected_path_to_curves (void)
 {
-	SPSelection * selection;
-	SPRepr * new;
-	SPItem * item;
-	SPPath * path;
-	SPCurve * curve;
-	gchar * str;
-	const gchar * transform, * style;
-	SPDesktop * desktop;
-	GSList * l, * sl = NULL, * nl = NULL;
-	
-	desktop = SP_ACTIVE_DESKTOP;
-	if (!SP_IS_DESKTOP(desktop)) return;
+	SPDesktop *dt;
+	SPItem *item;
+	SPCurve *curve;
 
-	selection = SP_DT_SELECTION (desktop);
+	dt = SP_ACTIVE_DESKTOP;
+	if (!dt) return;
+	item = sp_selection_item (SP_DT_SELECTION (dt));
+	if (!item) return;
 
-	sl = (GSList *) sp_selection_item_list (selection);
-	l = g_slist_copy (sl);
-	sp_selection_empty (selection);
+	if (SP_IS_PATH (item) && !SP_PATH (item)->independent) {
+		curve = sp_path_normalized_bpath (SP_PATH (item));
+	} else if (SP_IS_TEXT (item)) {
+		curve = sp_text_normalized_bpath (SP_TEXT (item));
+	} else {
+		curve = NULL;
+	}
 
-	for (; l != NULL; l = l->next) {
-		item = (SPItem *) l->data;
-		
-		if (item == NULL) continue;
-		if (!SP_IS_PATH (item)) continue;
+	if (curve) {
+		SPObject *parent;
+		SPRepr *new;
+		guchar *str;
 
-		path = SP_PATH (item);
-
-		curve = sp_path_normalized_bpath (path);
-		str = sp_svg_write_path (curve->bpath);
-		sp_curve_unref (curve);
-		transform = sp_repr_attr (SP_OBJECT (item)->repr, "transform");
-		style = sp_repr_attr (SP_OBJECT (item)->repr, "style");
+		parent = SP_OBJECT_PARENT (item);
 
 		new = sp_repr_new ("path");
-		sp_repr_set_attr (new, "transform", transform);
-		sp_repr_set_attr (new, "style", style);
-		sp_repr_set_attr (new, "d", str);
-
+		/* Transformation */
+		sp_repr_set_attr (new, "transform", sp_repr_attr (SP_OBJECT_REPR (item), "transform"));
+		/* Style */
+		str = sp_style_write_difference (SP_OBJECT_STYLE (item), SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
+		sp_repr_set_attr (new, "style", str);
 		g_free (str);
+		/* Definition */
+		str = sp_svg_write_path (curve->bpath);
+		sp_repr_set_attr (new, "d", str);
+		g_free (str);
+		sp_curve_unref (curve);
 
+		sp_repr_add_child (SP_OBJECT_REPR (parent), new, SP_OBJECT_REPR (item));
 		sp_repr_unparent (SP_OBJECT_REPR (item));
-		item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), new);
-		sp_repr_unref (new);
-		
-		nl = g_slist_append (nl, (gpointer)item);
-	}
-	sp_document_done (SP_DT_DOCUMENT (desktop));
 
-	sp_selection_set_item_list (selection, nl);
+		sp_document_done (SP_DT_DOCUMENT (dt));
+		sp_selection_set_repr (SP_DT_SELECTION (dt), new);
+		sp_repr_unref (new);
+	}
 }
+
+void
+sp_path_cleanup (SPPath *path)
+{
+	SPCurve *curve;
+	GSList *curves, *c;
+	SPStyle *style;
+	gboolean dropped;
+
+	if (strcmp (sp_repr_name (SP_OBJECT_REPR (path)), "path")) return;
+
+	style = SP_OBJECT_STYLE (path);
+	if (style->fill.type == SP_PAINT_TYPE_NONE) return;
+
+	curve = sp_path_normalized_bpath (path);
+	if (!curve) return;
+	c = sp_curve_split (curve);
+	sp_curve_unref (curve);
+
+	dropped = FALSE;
+	curves = NULL;
+	while (c) {
+		curve = (SPCurve *) c->data;
+		if (curve->closed) {
+			curves = g_slist_prepend (curves, curve);
+		} else {
+			dropped = TRUE;
+			sp_curve_unref (curve);
+		}
+		c = g_slist_remove (c, c->data);
+	}
+	curves = g_slist_reverse (curves);
+
+	curve = sp_curve_concat (curves);
+
+	while (curves) {
+		sp_curve_unref ((SPCurve *) curves->data);
+		curves = g_slist_remove (curves, curves->data);
+	}
+
+	if (sp_curve_is_empty (curve)) {
+		sp_repr_unparent (SP_OBJECT_REPR (path));
+	} else if (dropped) {
+		guchar *svgpath;
+		svgpath = sp_svg_write_path (curve->bpath);
+		sp_repr_set_attr (SP_OBJECT_REPR (path), "d", svgpath);
+		g_free (svgpath);
+	}
+
+	sp_curve_unref (curve);
+}
+
