@@ -1,6 +1,23 @@
-#define SP_SELECT_CONTEXT_C
+#define __SP_SELECT_CONTEXT_C__
 
+/*
+ * Selection and transformation context
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
+#include <config.h>
 #include <math.h>
+#include <string.h>
+#include <glib.h>
+#include <gdk/gdkkeysyms.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include "rubberband.h"
 #include "sodipodi-private.h"
 #include "document.h"
@@ -27,10 +44,13 @@ static void sp_select_context_class_init (SPSelectContextClass * klass);
 static void sp_select_context_init (SPSelectContext * select_context);
 static void sp_select_context_destroy (GtkObject * object);
 
-static void sp_select_context_setup (SPEventContext * event_context, SPDesktop * desktop);
+static void sp_select_context_setup (SPEventContext *ec);
+static void sp_select_context_set (SPEventContext *ec, const guchar *key, const guchar *val);
 static gint sp_select_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_select_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
 
+static GtkWidget *sp_select_context_config_widget (SPEventContext *ec);
+
 static void sp_selection_moveto (SPSelTrans * seltrans, double x, double y, guint state);
 
 static SPEventContextClass * parent_class;
@@ -76,8 +96,10 @@ sp_select_context_class_init (SPSelectContextClass * klass)
 	object_class->destroy = sp_select_context_destroy;
 
 	event_context_class->setup = sp_select_context_setup;
+	event_context_class->set = sp_select_context_set;
 	event_context_class->root_handler = sp_select_context_root_handler;
 	event_context_class->item_handler = sp_select_context_item_handler;
+	event_context_class->config_widget = sp_select_context_config_widget;
 
 	// cursors in select context
 	CursorSelectMouseover = sp_cursor_new_from_xpm (cursor_select_m_xpm , 1, 1); 
@@ -126,16 +148,41 @@ sp_select_context_destroy (GtkObject * object)
 }
 
 static void
-sp_select_context_setup (SPEventContext * event_context, SPDesktop * desktop)
+sp_select_context_setup (SPEventContext *ec)
 {
 	SPSelectContext * select_context;
 
-	select_context = SP_SELECT_CONTEXT (event_context);
+	select_context = SP_SELECT_CONTEXT (ec);
 
 	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (event_context, desktop);
+		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
 
-	sp_sel_trans_init (&select_context->seltrans, event_context->desktop);
+	sp_sel_trans_init (&select_context->seltrans, ec->desktop);
+
+	sp_event_context_read (ec, "show");
+	sp_event_context_read (ec, "transform");
+}
+
+static void
+sp_select_context_set (SPEventContext *ec, const guchar *key, const guchar *val)
+{
+	SPSelectContext *sc;
+
+	sc = SP_SELECT_CONTEXT (ec);
+
+	if (!strcmp (key, "show")) {
+		if (val && !strcmp (val, "outline")) {
+			sc->seltrans.show = SP_SELTRANS_SHOW_OUTLINE;
+		} else {
+			sc->seltrans.show = SP_SELTRANS_SHOW_CONTENT;
+		}
+	} else if (!strcmp (key, "transform")) {
+		if (val && !strcmp (val, "keep")) {
+			sc->seltrans.transform = SP_SELTRANS_TRANSFORM_KEEP;
+		} else {
+			sc->seltrans.transform = SP_SELTRANS_TRANSFORM_OPTIMIZE;
+		}
+	}
 }
 
 static gint
@@ -196,7 +243,7 @@ sp_select_context_item_handler (SPEventContext *event_context, SPItem *item, Gdk
 				// item has been moved
 				sp_sel_trans_ungrab (seltrans);
 				sc->moved = FALSE;
-				sp_desktop_clear_status (desktop);
+				sp_view_set_status (SP_VIEW (desktop), NULL, FALSE);
 			} else {
 				// item has not been moved -> do selecting
 				if (!sp_selection_is_empty (selection)) {
@@ -308,7 +355,7 @@ sp_select_context_root_handler (SPEventContext *event_context, GdkEvent * event)
 					// item has been moved
 					sp_sel_trans_ungrab (seltrans);
 					sc->moved = FALSE;
-					sp_desktop_clear_status (desktop);
+					sp_view_set_status (SP_VIEW (desktop), NULL, FALSE);
 				} else {
 					// item has not been moved -> do selecting
 					if (!sp_selection_is_empty (selection)) {
@@ -527,8 +574,87 @@ sp_selection_moveto (SPSelTrans * seltrans, double x, double y, guint state)
 	xs = SP_PT_TO_METRIC_STRING (dx, SP_DEFAULT_METRIC);
 	ys = SP_PT_TO_METRIC_STRING (dy, SP_DEFAULT_METRIC);
 	sprintf (status, "Move  %s, %s", xs->str, ys->str);
-	sp_desktop_set_status (seltrans->desktop, status);
+	sp_view_set_status (SP_VIEW (seltrans->desktop), status, FALSE);
 	g_string_free (xs, FALSE);
 	g_string_free (ys, FALSE);
 }
 
+/* Gtk styff */
+
+static void
+sp_select_context_show_toggled (GtkToggleButton *button, SPSelectContext *sc)
+{
+	if (gtk_toggle_button_get_active (button)) {
+		const gchar *val;
+		val = gtk_object_get_data (GTK_OBJECT (button), "value");
+		sp_repr_set_attr (SP_EVENT_CONTEXT_REPR (sc), "show", val);
+	}
+}
+
+static void
+sp_select_context_transform_toggled (GtkToggleButton *button, SPSelectContext *sc)
+{
+	if (gtk_toggle_button_get_active (button)) {
+		const gchar *val;
+		val = gtk_object_get_data (GTK_OBJECT (button), "value");
+		sp_repr_set_attr (SP_EVENT_CONTEXT_REPR (sc), "transform", val);
+	}
+}
+
+static GtkWidget *
+sp_select_context_config_widget (SPEventContext *ec)
+{
+	SPSelectContext *sc;
+	GtkWidget *vb, *f, *fb, *b;
+
+	sc = SP_SELECT_CONTEXT (ec);
+
+	vb = gtk_vbox_new (FALSE, 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
+
+	f = gtk_frame_new (_("Visual transformation"));
+	gtk_widget_show (f);
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+
+	fb = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (fb);
+	gtk_container_add (GTK_CONTAINER (f), fb);
+
+	b = gtk_radio_button_new_with_label (NULL, _("Show content"));
+	gtk_widget_show (b);
+	gtk_object_set_data (GTK_OBJECT (b), "value", "content");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.show == SP_SELTRANS_SHOW_CONTENT);
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_show_toggled), sc);
+
+	b = gtk_radio_button_new_with_label (gtk_radio_button_group (GTK_RADIO_BUTTON (b)), _("Show outline"));
+	gtk_widget_show (b);
+	gtk_object_set_data (GTK_OBJECT (b), "value", "outline");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.show == SP_SELTRANS_SHOW_OUTLINE);
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_show_toggled), sc);
+
+	f = gtk_frame_new (_("Object transformation"));
+	gtk_widget_show (f);
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+
+	fb = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (fb);
+	gtk_container_add (GTK_CONTAINER (f), fb);
+
+	b = gtk_radio_button_new_with_label (NULL, _("Optimize"));
+	gtk_widget_show (b);
+	gtk_object_set_data (GTK_OBJECT (b), "value", "optimize");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.transform == SP_SELTRANS_TRANSFORM_OPTIMIZE);
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_transform_toggled), sc);
+
+	b = gtk_radio_button_new_with_label (gtk_radio_button_group (GTK_RADIO_BUTTON (b)), _("Preserve"));
+	gtk_widget_show (b);
+	gtk_object_set_data (GTK_OBJECT (b), "value", "keep");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), sc->seltrans.transform == SP_SELTRANS_TRANSFORM_KEEP);
+	gtk_box_pack_start (GTK_BOX (fb), b, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (sp_select_context_transform_toggled), sc);
+
+	return vb;
+}
