@@ -1,12 +1,16 @@
-#define SP_SELECTION_CHEMISTRY_C
+#define __SP_SELECTION_CHEMISTRY_C__
 
 /*
- * sp-selection
+ * Miscellanous operations on selected items
  *
- * selection handling functions
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Frank Felfe <innerspace@iname.com>
  *
- * TODO: ungrouping destroys group attributes - should inherit to children
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <string.h>
@@ -16,12 +20,15 @@
 #include "sodipodi.h"
 #include "desktop.h"
 #include "desktop-handles.h"
-#include "selection-chemistry.h"
 #include "sp-item-transform.h" 
 #include "sp-item-group.h"
+#include "sp-path.h"
+#include "path-chemistry.h"
+
+#include "selection-chemistry.h"
 
 /* fixme: find a better place */
-GSList * clipboard = NULL;
+GSList *clipboard = NULL;
 
 void
 sp_selection_delete (GtkWidget * widget)
@@ -102,6 +109,82 @@ void sp_selection_duplicate (GtkWidget * widget)
 	g_slist_free (newsel);
 }
 
+void
+sp_edit_clear_all (gpointer data)
+{
+	SPDesktop *dt;
+	SPDocument *doc;
+	GSList *items;
+
+	dt = SP_ACTIVE_DESKTOP;
+	if (!dt) return;
+	doc = SP_DT_DOCUMENT (dt);
+	sp_selection_set_empty (SP_DT_SELECTION (dt));
+
+	items = sp_item_group_item_list (SP_GROUP (sp_document_root (doc)));
+
+	while (items) {
+		sp_repr_unparent (SP_OBJECT_REPR (items->data));
+		items = g_slist_remove (items, items->data);
+	}
+
+	sp_document_done (doc);
+}
+
+static void
+sp_group_cleanup (SPGroup *group)
+{
+	SPObject *child;
+	GSList *l;
+
+	l = NULL;
+	for (child = group->children; child != NULL; child = child->next) {
+		sp_object_ref (child, NULL);
+		l = g_slist_prepend (l, child);
+	}
+
+	while (l) {
+		if (SP_IS_GROUP (l->data)) {
+			sp_group_cleanup (SP_GROUP (l->data));
+		} else if (SP_IS_PATH (l->data)) {
+			sp_path_cleanup (SP_PATH (l->data));
+		}
+		sp_object_unref (SP_OBJECT (l->data), NULL);
+		l = g_slist_remove (l, l->data);
+	}
+
+
+	if (!strcmp (sp_repr_name (SP_OBJECT_REPR (group)), "g")) {
+		gint numitems;
+		numitems = 0;
+		for (child = group->children; child != NULL; child = child->next) {
+			if (SP_IS_ITEM (child)) numitems += 1;
+		}
+		if (numitems <= 1) {
+			sp_item_group_ungroup (group, NULL);
+		}
+	}
+}
+
+void
+sp_edit_cleanup (gpointer data)
+{
+	SPDocument *doc;
+	SPGroup *root;
+
+	doc = SP_ACTIVE_DOCUMENT;
+	if (!doc) return;
+	if (SP_ACTIVE_DESKTOP) {
+		sp_selection_empty (SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
+	}
+
+	root = SP_GROUP (SP_DOCUMENT_ROOT (doc));
+
+	sp_group_cleanup (root);
+
+	sp_document_done (doc);
+}
+
 /* fixme: sequencing */
 
 void
@@ -152,94 +235,92 @@ sp_selection_group (GtkWidget * widget)
 	sp_repr_unref (group);
 }
 
-void sp_selection_ungroup (GtkWidget * widget)
+void
+sp_selection_ungroup (gpointer data)
 {
-	SPDesktop * desktop;
-	SPSelection * selection;
-	SPItem * item;
-	SPRepr * current;
-	SPRepr * child;
-	const gchar * pastr, * castr;
-	gdouble pa[6], ca[6];
-	SPCSSAttr * css;
-	gchar affinestr[80];
+	SPDesktop *dt;
+	SPItem *group;
+	GSList *children;
 
-	desktop = SP_ACTIVE_DESKTOP;
-	if (desktop == NULL) return;
-	selection = SP_DT_SELECTION (desktop);
-	if (sp_selection_is_empty (selection)) return;
-	current = sp_selection_repr (selection);
-	if (current == NULL) return;
-	if (strcmp (sp_repr_name (current), "g") != 0) return;
-	sp_selection_empty (selection);
+	dt = SP_ACTIVE_DESKTOP;
+	if (!dt) return;
+	group = sp_selection_item (SP_DT_SELECTION (dt));
+	if (!group) return;
+	/* We do not allow ungrouping <svg> etc. */
+	if (strcmp (sp_repr_name (SP_OBJECT_REPR (group)), "g")) return;
 
-	art_affine_identity (pa);
-	pastr = sp_repr_attr (current, "transform");
-	sp_svg_read_affine (pa, pastr);
+	children = NULL;
+	/* This is not strictly required, but is nicer to rely on group ::destroy */
+	sp_selection_empty (SP_DT_SELECTION (dt));
+	sp_item_group_ungroup (SP_GROUP (group), &children);
+	sp_selection_set_item_list (SP_DT_SELECTION (dt), children);
+	g_slist_free (children);
+}
 
-	while (current->children) {
-		child = current->children;
+static SPGroup *
+sp_item_list_common_parent_group (const GSList *items)
+{
+	SPObject *parent;
 
-		art_affine_identity (ca);
-		castr = sp_repr_attr (child, "transform");
-		sp_svg_read_affine (ca, castr);
-		art_affine_multiply (ca, ca, pa);
-
-		css = sp_repr_css_attr_inherited (child, "style");
-		sp_repr_ref (child);
-		sp_repr_remove_child (current, child);
-		
-		sp_svg_write_affine (affinestr, 79, ca);
-		affinestr[79] = '\0';
-		sp_repr_set_attr (child, "transform", affinestr);
-
-		sp_repr_css_set (child, css, "style");
-		sp_repr_css_attr_unref (css);
-
-		item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), child);
-		sp_repr_unref (child);
-		if (SP_IS_ITEM (item)) sp_selection_add_item (selection, item);
+	if (!items) return NULL;
+	parent = SP_OBJECT_PARENT (items->data);
+	/* Strictly speaking this CAN happen, if user selects <svg> from XML editor */
+	if (!SP_IS_GROUP (parent)) return NULL;
+	for (items = items->next; items; items = items->next) {
+		if (SP_OBJECT_PARENT (items->data) != parent) return NULL;
 	}
 
-	sp_repr_unparent (current);
-	sp_document_done (SP_DT_DOCUMENT (desktop));
+	return SP_GROUP (parent);
 }
 
+#if 0
+#define PRINT_STR(s) g_print (s)
+#define PRINT_OBJ(s, o) g_print ("%s: %s\n", s, (o) ? (gchar *) sp_repr_attr (SP_OBJECT_REPR (o), "id") : "NULL")
+#else
+#define PRINT_STR(s)
+#define PRINT_OBJ(s, o)
+#endif
+
 void sp_selection_raise (GtkWidget * widget)
 {
-	SPDocument * document;
-	SPSelection * selection;
-	SPDesktop * desktop;
-	SPRepr * repr;
-	GSList * rl;
-	GSList * l;
-	GSList * pl;
+	SPDesktop *dt;
+	const GSList *items;
+	SPGroup *group;
+	SPRepr *grepr;
+	SPObject *child, *newref;
+	GSList *rev;
 
-	desktop = SP_ACTIVE_DESKTOP;
-	if (desktop == NULL) return;
-	document = SP_DT_DOCUMENT (desktop);
-	selection = SP_DT_SELECTION (desktop);
+	dt = SP_ACTIVE_DESKTOP;
+	if (!dt) return;
+	items = sp_selection_item_list (SP_DT_SELECTION (dt));
+	if (!items) return;
+	group = sp_item_list_common_parent_group (items);
+	if (!group) return;
+	grepr = SP_OBJECT_REPR (group);
 
-	if (sp_selection_is_empty (selection)) return;
-
-	rl = g_slist_copy ((GSList *) sp_selection_repr_list (selection));
-
-	pl = NULL;
-
-	for (l = rl; l != NULL; l = l->next) {
-		repr = (SPRepr *) l->data;
-		pl = g_slist_append (pl, GINT_TO_POINTER (sp_repr_position (repr)));
+	/* construct reverse-ordered list of selected children */
+	rev = NULL;
+	for (child = group->children; child; child = child->next) {
+		if (g_slist_find ((GSList *) items, child)) {
+			rev = g_slist_prepend (rev, child);
+		}
 	}
 
-	for (l = rl; l != NULL; l = l->next) {
-		repr = (SPRepr *) l->data;
-		sp_repr_set_position_absolute (repr, GPOINTER_TO_INT (pl->data) + 1);
-		pl = g_slist_remove (pl, pl->data);
+	while (rev) {
+		child = SP_OBJECT (rev->data);
+		for (newref = child->next; newref; newref = newref->next) {
+			if (SP_IS_ITEM (newref)) {
+				if (!g_slist_find ((GSList *) items, newref)) {
+					/* Found available position */
+					sp_repr_change_order (grepr, SP_OBJECT_REPR (child), SP_OBJECT_REPR (newref));
+				}
+				break;
+			}
+		}
+		rev = g_slist_remove (rev, child);
 	}
 
-	g_slist_free (rl);
-
-	sp_document_done (document);
+	sp_document_done (SP_DT_DOCUMENT (dt));
 }
 
 void sp_selection_raise_to_top (GtkWidget * widget)
@@ -270,54 +351,89 @@ void sp_selection_raise_to_top (GtkWidget * widget)
 	sp_document_done (document);
 }
 
-void sp_selection_lower (GtkWidget * widget)
+void
+sp_selection_lower (GtkWidget *widget)
 {
-	SPDocument * document;
-	SPSelection * selection;
-	SPDesktop * desktop;
-	SPRepr * repr;
-	GSList * rl;
-	GSList * l;
-	GSList * pl;
-	gint pos;
+	SPDesktop *dt;
+	const GSList *items;
+	SPGroup *group;
+	SPRepr *grepr;
+	SPObject *child, *newref, *oldref;
+	gboolean skip;
 
-	desktop = SP_ACTIVE_DESKTOP;
-	if (desktop == NULL) return;
-	document = SP_DT_DOCUMENT (desktop);
-	selection = SP_DT_SELECTION (desktop);
+	dt = SP_ACTIVE_DESKTOP;
+	if (!dt) return;
+	items = sp_selection_item_list (SP_DT_SELECTION (dt));
+	if (!items) return;
+	group = sp_item_list_common_parent_group (items);
+	if (!group) return;
+	grepr = SP_OBJECT_REPR (group);
 
-	if (sp_selection_is_empty (selection)) return;
-
-	rl = g_slist_copy ((GSList *) sp_selection_repr_list (selection));
-
-	pl = NULL;
-
-	for (l = rl; l != NULL; l = l->next) {
-		repr = (SPRepr *) l->data;
-		pl = g_slist_append (pl, GINT_TO_POINTER (sp_repr_position (repr)));
-	}
-
-	for (l = rl; l != NULL; l = l->next) {
-		gint minpos;
-		SPObject *pp, *pc;
-		repr = (SPRepr *) l->data;
-		pp = sp_document_lookup_id (document, sp_repr_attr (sp_repr_parent (repr), "id"));
-		minpos = 0;
-		g_assert (SP_IS_GROUP (pp));
-		pc = SP_GROUP (pp)->children;
-		while (!SP_IS_ITEM (pc)) {
-			minpos += 1;
-			pc = pc->next;
+	PRINT_STR ("STARTING\n");
+	/* Start from beginning */
+	skip = TRUE;
+	newref = NULL;
+	oldref = NULL;
+	child = group->children;
+	while (child != NULL) {
+		if (SP_IS_ITEM (child)) {
+			/* We are item */
+			skip = FALSE;
+			/* fixme: Remove from list (Lauris) */
+			if (g_slist_find ((GSList *) items, child)) {
+				/* Need lower */
+				if (newref != oldref) {
+					if (sp_repr_change_order (grepr, SP_OBJECT_REPR (child), (newref) ? SP_OBJECT_REPR (newref) : NULL)) {
+						PRINT_STR ("Change order succeeded\n");
+						PRINT_OBJ ("  child", child);
+						PRINT_OBJ ("  oldref", oldref);
+						PRINT_OBJ ("  newref", newref);
+						/* Order change succeeded */
+						/* Next available position */
+						newref = child;
+						/* Oldref is just what it was */
+						/* Continue from oldref */
+						child = oldref->next;
+					} else {
+						PRINT_STR ("Change order failed\n");
+						PRINT_OBJ ("  child", child);
+						PRINT_OBJ ("  oldref", oldref);
+						PRINT_OBJ ("  newref", newref);
+						/* Order change did not succeed */
+						newref = oldref;
+						oldref = child;
+						child = child->next;
+					}
+				} else {
+					/* Item position will not change */
+					/* Other items will lower only following positions */
+					newref = child;
+					oldref = child;
+					child = child->next;
+				}
+			} else {
+				PRINT_STR ("Item not in list\n");
+				PRINT_OBJ ("  child", child);
+				PRINT_OBJ ("  oldref", oldref);
+				PRINT_OBJ ("  newref", newref);
+				/* We were item, but not in list */
+				newref = oldref;
+				oldref = child;
+				child = child->next;
+			}
+		} else {
+			PRINT_STR ("Not an item\n");
+			PRINT_OBJ ("  child", child);
+			PRINT_OBJ ("  oldref", oldref);
+			PRINT_OBJ ("  newref", newref);
+			/* We want to refind newref only to skip initial non-items */
+			if (skip) newref = child;
+			oldref = child;
+			child = child->next;
 		}
-		pos = GPOINTER_TO_INT (pl->data) - 1;
-		if (pos < minpos) pos = minpos;
-		sp_repr_set_position_absolute (repr, pos);
-		pl = g_slist_remove (pl, pl->data);
 	}
 
-	g_slist_free (rl);
-
-	sp_document_done (document);
+	sp_document_done (SP_DT_DOCUMENT (dt));
 }
 
 void sp_selection_lower_to_bottom (GtkWidget * widget)
@@ -636,67 +752,72 @@ sp_selection_move_screen (gdouble sx, gdouble sy)
   sp_document_done (SP_DT_DOCUMENT (desktop));
 }
 
-
 void
 sp_selection_item_next (void)
 {
-  SPDocument * document;
-  SPDesktop * desktop;
-  SPSelection * selection;
-  SPItem * item = NULL;
-  GSList * children = NULL, * l = NULL;
-  ArtDRect dbox,sbox;
-  ArtPoint s,d;
-  gint dx=0, dy=0;
+	SPDocument *document;
+	SPDesktop *desktop;
+	SPSelection *selection;
+	SPItem *item = NULL;
+	GSList *children = NULL, *l = NULL;
+	ArtDRect dbox,sbox;
+	ArtPoint s,d;
+	gint dx=0, dy=0;
 
-  document = SP_ACTIVE_DOCUMENT;
-  desktop = SP_ACTIVE_DESKTOP;
-  g_return_if_fail(document != NULL);
-  g_return_if_fail(desktop != NULL);
-  if (!SP_IS_DESKTOP (desktop)) return;
-  selection = SP_DT_SELECTION(desktop);
-  g_return_if_fail(selection!=NULL);
+	document = SP_ACTIVE_DOCUMENT;
+	desktop = SP_ACTIVE_DESKTOP;
+	g_return_if_fail(document != NULL);
+	g_return_if_fail(desktop != NULL);
+	if (!SP_IS_DESKTOP (desktop)) return;
+	selection = SP_DT_SELECTION(desktop);
+	g_return_if_fail(selection!=NULL);
   
-  // get item list
-  if (SP_CYCLING == SP_CYCLE_VISIBLE) {
-    sp_desktop_get_visible_area (desktop, &dbox);
-    children = sp_document_items_in_box (document, &dbox);
-  } else {
-    children = sp_item_group_item_list (SP_GROUP(sp_document_root(document)));
-  }
+	// get item list
+	if (SP_CYCLING == SP_CYCLE_VISIBLE) {
+		sp_desktop_get_visible_area (desktop, &dbox);
+		children = sp_document_items_in_box (document, &dbox);
+	} else {
+		children = sp_item_group_item_list (SP_GROUP(sp_document_root(document)));
+	}
 
-  // compute next item
-  if (children == NULL) return;
-  if sp_selection_is_empty(selection) item = children->data;
-  else {
-    l = g_slist_find(children,selection->items->data);
-    if ((l == NULL) || (l->next == NULL)) item = SP_ITEM (children->data);
-    else item = SP_ITEM (l->next->data);
-  }
+	// compute next item
+	if (children == NULL) return;
+	if sp_selection_is_empty(selection) {
+		item = children->data;
+	} else {
+		l = g_slist_find(children,selection->items->data);
+		if ((l == NULL) || (l->next == NULL)) {
+			item = SP_ITEM (children->data);
+		} else {
+			item = SP_ITEM (l->next->data);
+		}
+	}
 
-  // set selection to item
-  if (item != NULL) sp_selection_set_item (selection, item);
-  else return;
-  
-  g_slist_free (children);
+	// set selection to item
+	if (item != NULL) {
+		sp_selection_set_item (selection, item);
+	} else {
+		return;
+	}
 
-  // adjust visible area to see whole new selection
-  if (SP_CYCLING == SP_CYCLE_FOCUS) {
-    sp_desktop_get_visible_area (desktop, &dbox);
-    sp_item_bbox (item, &sbox);
-    if (dbox.x0>sbox.x0 || dbox.y0>sbox.y0 || dbox.x1<sbox.x1 || dbox.y1<sbox.y1 ) {
-      s.x = (sbox.x0+sbox.x1)/2;
-      s.y = (sbox.y0+sbox.y1)/2;
-      d.x = (dbox.x0+dbox.x1)/2;
-      d.y = (dbox.y0+dbox.y1)/2;
-      art_affine_point (&s, &s, desktop->d2w);
-      art_affine_point (&d, &d, desktop->d2w);
-      dx = (gint)(d.x-s.x);
-      dy = (gint)(d.y-s.y);
-      sp_desktop_scroll_world (desktop, dx, dy);
-    }
-  }
+	g_slist_free (children);
 
+	// adjust visible area to see whole new selection
+	if (SP_CYCLING == SP_CYCLE_FOCUS) {
+		sp_desktop_get_visible_area (desktop, &dbox);
+		sp_item_bbox_desktop (item, &sbox);
+		if (dbox.x0 > sbox.x0 || dbox.y0 > sbox.y0 || dbox.x1 < sbox.x1 || dbox.y1 < sbox.y1 ) {
+			s.x = (sbox.x0 + sbox.x1) / 2;
+			s.y = (sbox.y0 + sbox.y1) / 2;
+			d.x = (dbox.x0 + dbox.x1) / 2;
+			d.y = (dbox.y0 + dbox.y1) / 2;
+			art_affine_point (&s, &s, desktop->d2w);
+			art_affine_point (&d, &d, desktop->d2w);
+			dx = (gint) (d.x-s.x);
+			dy = (gint) (d.y-s.y);
+			sp_desktop_scroll_world (desktop, dx, dy);
+		}
+	}
 }
 
 void
@@ -748,7 +869,7 @@ sp_selection_item_prev (void)
   // adjust visible area to see whole new selection
   if (SP_CYCLING == SP_CYCLE_FOCUS) {
     sp_desktop_get_visible_area (desktop, &dbox);
-    sp_item_bbox (item, &sbox);
+    sp_item_bbox_desktop (item, &sbox);
     if (dbox.x0>sbox.x0 || dbox.y0>sbox.y0 || dbox.x1<sbox.x1 || dbox.y1<sbox.y1 ) {
       s.x = (sbox.x0+sbox.x1)/2;
       s.y = (sbox.y0+sbox.y1)/2;
