@@ -1,12 +1,15 @@
-#define SODIPODI_C
+#define __SODIPODI_C__
 
 /*
- * Sodipodi
+ * Interface to main application
  *
- * This is interface to main application
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) Lauris Kaplinski <lauris@kaplinski.com> 1999-2000
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <config.h>
@@ -19,6 +22,7 @@
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <xml/repr-private.h>
 #include "document.h"
@@ -115,7 +119,7 @@ sodipodi_class_init (SodipodiClass * klass)
 		object_class->type,
 		GTK_SIGNAL_OFFSET (SodipodiClass, modify_selection),
 		gtk_marshal_NONE__POINTER_UINT,
-		GTK_TYPE_NONE, 1,
+		GTK_TYPE_NONE, 2,
 		GTK_TYPE_POINTER, GTK_TYPE_UINT);
 	sodipodi_signals[CHANGE_SELECTION] = gtk_signal_new ("change_selection",
 		GTK_RUN_LAST,
@@ -232,54 +236,140 @@ sodipodi_desactivate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop)
 
 /* fixme: This is EVIL, and belongs to main after all */
 
+#define SP_INDENT 8
+
 static void
 sodipodi_segv_handler (int signum)
 {
 	static gint recursion = FALSE;
-	GSList *l;
-	gchar *home;
-	gint count, date;
+	GSList *savednames, *failednames, *l;
+	gchar *home, *istr, *sstr, *fstr, *b;
+	gint count, nllen, len, pos;
+	time_t sptime;
+	struct tm *sptm;
+	char sptstr[256];
+	GtkWidget *msgbox;
 
+	/* Kill loops */
 	if (recursion) abort ();
 	recursion = TRUE;
 
 	g_warning ("Emergency save activated");
 
 	home = g_get_home_dir ();
-	date = time (NULL);
+	sptime = time (NULL);
+	sptm = localtime (&sptime);
+	strftime (sptstr, 256, "%Y_%m_%d_%H_%M_%S", sptm);
 
 	count = 0;
+	savednames = NULL;
+	failednames = NULL;
 	for (l = sodipodi->documents; l != NULL; l = l->next) {
 		SPDocument *doc;
 		SPRepr *repr;
 		doc = (SPDocument *) l->data;
 		repr = sp_document_repr_root (doc);
 		if (sp_repr_attr (repr, "sodipodi:modified")) {
-			gchar *path;
+			const guchar *docname;
+			gchar c[1024];
 			FILE *file;
-			path = g_strdup_printf ("%s/.sodipodi/emergency-%d-%d.svg", home, date, count);
-			file = fopen (path, "w");
-			g_free (path);
+			docname = sp_repr_attr (repr, "sodipodi:docname");
+			if (docname) {
+				docname = g_basename (docname);
+			}
+			if (!docname || !*docname) docname = "emergency";
+			g_snprintf (c, 1024, "%s/.sodipodi/%.256s.%s.%d", home, docname, sptstr, count);
+			file = fopen (c, "w");
 			if (!file) {
-				path = g_strdup_printf ("%s/sodipodi-emergency-%d-%d.svg", home, date, count);
-				file = fopen (path, "w");
-				g_free (path);
+				g_snprintf (c, 1024, "%s/sodipodi-%.256s.%s.%d", home, docname, sptstr, count);
+				file = fopen (c, "w");
 			}
 			if (!file) {
-				path = g_strdup_printf ("/tmp/sodipodi-emergency-%d-%d.svg", date, count);
-				file = fopen (path, "w");
-				g_free (path);
+				g_snprintf (c, 1024, "/tmp/sodipodi-%.256s.%s.%d", docname, sptstr, count);
+				file = fopen (c, "w");
 			}
 			if (file) {
 				sp_repr_save_stream (sp_repr_document (repr), file);
+				savednames = g_slist_prepend (savednames, g_strdup (c));
 				fclose (file);
+			} else {
+				docname = sp_repr_attr (repr, "sodipodi:docname");
+				failednames = g_slist_prepend (failednames, (docname) ? g_strdup (docname) : g_strdup (_("Untitled document")));
 			}
 			count++;
 		}
 	}
 
+	savednames = g_slist_reverse (savednames);
+	failednames = g_slist_reverse (failednames);
+	if (savednames) {
+		fprintf (stderr, "\nEmergency save locations:\n");
+		for (l = savednames; l != NULL; l = l->next) {
+			fprintf (stderr, "  %s\n", (gchar *) l->data);
+		}
+	}
+	if (failednames) {
+		fprintf (stderr, "Failed to do emergency save for:\n");
+		for (l = failednames; l != NULL; l = l->next) {
+			fprintf (stderr, "  %s\n", (gchar *) l->data);
+		}
+	}
+
 	g_warning ("Emergency save completed, now crashing...");
 
+	/* Show nice dialog box */
+
+	istr = N_("Sodipodi encountered an internal error and will close now.\n");
+	sstr = N_("Automatic backups of unsaved documents were done to following locations:\n");
+	fstr = N_("Automatic backup of following documents failed:\n");
+	nllen = strlen ("\n");
+	len = strlen (istr) + strlen (sstr) + strlen (fstr);
+	for (l = savednames; l != NULL; l = l->next) {
+		len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
+	}
+	for (l = failednames; l != NULL; l = l->next) {
+		len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
+	}
+	len += 1;
+	b = g_new (gchar, len);
+	pos = 0;
+	len = strlen (istr);
+	memcpy (b + pos, istr, len);
+	pos += len;
+	if (savednames) {
+		len = strlen (sstr);
+		memcpy (b + pos, sstr, len);
+		pos += len;
+		for (l = savednames; l != NULL; l = l->next) {
+			memset (b + pos, ' ', SP_INDENT);
+			pos += SP_INDENT;
+			len = strlen ((gchar *) l->data);
+			memcpy (b + pos, l->data, len);
+			pos += len;
+			memcpy (b + pos, "\n", nllen);
+			pos += nllen;
+		}
+	}
+	if (failednames) {
+		len = strlen (fstr);
+		memcpy (b + pos, fstr, len);
+		pos += len;
+		for (l = failednames; l != NULL; l = l->next) {
+			memset (b + pos, ' ', SP_INDENT);
+			pos += SP_INDENT;
+			len = strlen ((gchar *) l->data);
+			memcpy (b + pos, l->data, len);
+			pos += len;
+			memcpy (b + pos, "\n", nllen);
+			pos += nllen;
+		}
+	}
+	*(b + pos) = '\0';
+	msgbox = gnome_message_box_new (b, GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_CLOSE, NULL);
+	gnome_dialog_run_and_close (GNOME_DIALOG (msgbox));
+	gtk_widget_destroy (msgbox);
+	g_free (b);
+
 	(* segv_handler) (signum);
 }
 
@@ -359,7 +449,7 @@ sodipodi_load_preferences (Sodipodi * sodipodi)
 		return;
 	}
 
-	sp_repr_document_overwrite (sodipodi->preferences, doc, "id");
+	sp_repr_document_merge (sodipodi->preferences, doc, "id");
 	sp_repr_document_unref (doc);
 	g_free (fn);
 }
@@ -411,15 +501,14 @@ sodipodi_get_repr (Sodipodi * sodipodi, const gchar * key)
 }
 
 void
-sodipodi_selection_modified (SPSelection *selection)
+sodipodi_selection_modified (SPSelection *selection, guint flags)
 {
 	g_return_if_fail (sodipodi != NULL);
 	g_return_if_fail (selection != NULL);
 	g_return_if_fail (SP_IS_SELECTION (selection));
 
 	if (DESKTOP_IS_ACTIVE (selection->desktop)) {
-		gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[MODIFY_SELECTION], selection,
-				 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+		gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[MODIFY_SELECTION], selection, flags);
 	}
 }
 
