@@ -1,18 +1,15 @@
 #define __SP_GRADIENT_C__
 
 /*
- * SPGradient
- *
- * TODO: Implement radial & other fancy gradients
- * TODO: Implement linking attributes
+ * SVG <stop> <linearGradient> and <radialGradient> implementation
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
- * Released under GNU GPL
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <math.h>
@@ -20,12 +17,18 @@
 #include <stdlib.h>
 #include <libart_lgpl/art_affine.h>
 #include <gtk/gtksignal.h>
+
 #include "svg/svg.h"
 #include "xml/repr-private.h"
+#include "helper/nr-plain-stuff.h"
 #include "document-private.h"
 #include "sp-object-repr.h"
 #include "sp-gradient.h"
 
+#define SP_MACROS_SILENT
+#include "macros.h"
+
+/* Has to be power of 2 */
 #define NCOLORS 1024
 
 static void sp_stop_class_init (SPStopClass * klass);
@@ -39,9 +42,9 @@ static SPObjectClass * stop_parent_class;
 GtkType
 sp_stop_get_type (void)
 {
-	static GtkType stop_type = 0;
-	if (!stop_type) {
-		GtkTypeInfo stop_info = {
+	static GtkType type = 0;
+	if (!type) {
+		GtkTypeInfo info = {
 			"SPStop",
 			sizeof (SPStop),
 			sizeof (SPStopClass),
@@ -49,9 +52,9 @@ sp_stop_get_type (void)
 			(GtkObjectInitFunc) sp_stop_init,
 			NULL, NULL, NULL
 		};
-		stop_type = gtk_type_unique (sp_object_get_type (), &stop_info);
+		type = gtk_type_unique (SP_TYPE_OBJECT, &info);
 	}
-	return stop_type;
+	return type;
 }
 
 static void
@@ -61,7 +64,7 @@ sp_stop_class_init (SPStopClass * klass)
 
 	sp_object_class = (SPObjectClass *) klass;
 
-	stop_parent_class = gtk_type_class (sp_object_get_type ());
+	stop_parent_class = gtk_type_class (SP_TYPE_OBJECT);
 
 	sp_object_class->build = sp_stop_build;
 	sp_object_class->read_attr = sp_stop_read_attr;
@@ -81,6 +84,8 @@ sp_stop_build (SPObject * object, SPDocument * document, SPRepr * repr)
 		(* SP_OBJECT_CLASS (stop_parent_class)->build) (object, document, repr);
 
 	sp_stop_read_attr (object, "offset");
+	sp_stop_read_attr (object, "stop-color");
+	sp_stop_read_attr (object, "stop-opacity");
 	sp_stop_read_attr (object, "style");
 }
 
@@ -93,31 +98,40 @@ sp_stop_read_attr (SPObject * object, const gchar * key)
 
 	stop = SP_STOP (object);
 
-	if (strcmp (key, "style") == 0) {
+	if (!strcmp (key, "style")) {
 		const guchar *p;
+		/* fixme: We are reading simple values 3 times during build (Lauris) */
+		/* fixme: We need presentation attributes etc. */
 		p = sp_object_get_style_property (object, "stop-color", "black");
-		color = sp_svg_read_color (p, 0x00000000);
+		color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
 		sp_color_set_rgb_rgba32 (&stop->color, color);
 		p = sp_object_get_style_property (object, "stop-opacity", "1");
-		opacity = sp_svg_read_percentage (p, 1.0);
+		opacity = sp_svg_read_percentage (p, stop->opacity);
 		stop->opacity = opacity;
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+	} else if (!strcmp (key, "stop-color")) {
+		const guchar *p;
+		/* fixme: We need presentation attributes etc. */
+		p = sp_object_get_style_property (object, "stop-color", "black");
+		color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
+		sp_color_set_rgb_rgba32 (&stop->color, color);
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+	} else if (!strcmp (key, "stop-opacity")) {
+		const guchar *p;
+		/* fixme: We need presentation attributes etc. */
+		p = sp_object_get_style_property (object, "stop-opacity", "1");
+		opacity = sp_svg_read_percentage (p, stop->opacity);
+		stop->opacity = opacity;
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+	} else if (!strcmp (key, "offset")) {
+		const guchar *str;
+		str = sp_repr_attr (object->repr, key);
+		stop->offset = sp_svg_read_percentage (str, 0.0);
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
+	} else {
+		if (SP_OBJECT_CLASS (stop_parent_class)->read_attr)
+			(* SP_OBJECT_CLASS (stop_parent_class)->read_attr) (object, key);
 	}
-	if (strcmp (key, "offset") == 0) {
-		static const SPUnit *percent = NULL;
-		const SPUnit *unit;
-		const guchar *val;
-		if (!percent) percent = sp_unit_get_by_abbreviation ("%");
-		val = sp_repr_attr (object->repr, key);
-		stop->offset = sp_svg_read_length (&unit, val, 0.0);
-		if (unit == percent) stop->offset /= 100.0;
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
-	}
-
-	if (SP_OBJECT_CLASS (stop_parent_class)->read_attr)
-		(* SP_OBJECT_CLASS (stop_parent_class)->read_attr) (object, key);
 }
 
 /*
@@ -454,7 +468,7 @@ sp_gradient_modified (SPObject *object, guint flags)
 	}
 
 	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
-	flags &= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
 
 	l = NULL;
 	for (child = gr->stops; child != NULL; child = child->next) {
@@ -488,8 +502,11 @@ sp_gradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set_missi
 
 	if (set_missing || gr->transform_set) {
 		gchar c[256];
-		sp_svg_write_affine (c, 256, gr->transform);
-		sp_repr_set_attr (repr, "gradientTransform", c);
+		if (sp_svg_write_affine (c, 256, gr->transform)) {
+			sp_repr_set_attr (repr, "gradientTransform", c);
+		} else {
+			sp_repr_set_attr (repr, "gradientTransform", NULL);
+		}
 	}
 
 	if (set_missing || gr->spread_set) {
@@ -970,7 +987,6 @@ struct _SPLGPainter {
 	gdouble len;
 	gdouble x0, y0;
 	gdouble dx, dy;
-	guint32 opacity;
 };
 
 static void sp_lineargradient_class_init (SPLinearGradientClass * klass);
@@ -980,7 +996,7 @@ static void sp_lineargradient_destroy (GtkObject * object);
 static void sp_lineargradient_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_lineargradient_read_attr (SPObject * object, const gchar * key);
 
-static SPPainter *sp_lineargradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opacity, ArtDRect *bbox);
+static SPPainter *sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox);
 static void sp_lineargradient_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static void sp_lineargradient_flatten_attributes (SPGradient *gradient, SPRepr *repr, gboolean set_missing);
@@ -1036,14 +1052,10 @@ sp_lineargradient_class_init (SPLinearGradientClass * klass)
 static void
 sp_lineargradient_init (SPLinearGradient * lg)
 {
-	lg->x1.distance = 0.0;
-	lg->x1.unit = sp_unit_get_by_abbreviation ("%");
-	lg->y1.distance = 0.0;
-	lg->y1.unit = sp_unit_get_by_abbreviation ("%");
-	lg->x2.distance = 100.0;
-	lg->x2.unit = sp_unit_get_by_abbreviation ("%");
-	lg->y2.distance = 0.0;
-	lg->y2.unit = sp_unit_get_by_abbreviation ("%");
+	sp_svg_length_unset (&lg->x1, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
+	sp_svg_length_unset (&lg->y1, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
+	sp_svg_length_unset (&lg->x2, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
+	sp_svg_length_unset (&lg->y2, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
 }
 
 static void
@@ -1077,51 +1089,61 @@ static void
 sp_lineargradient_read_attr (SPObject * object, const gchar * key)
 {
 	SPLinearGradient * lg;
-	const gchar *val;
+	const guchar *str;
 
 	lg = SP_LINEARGRADIENT (object);
 
-	val = sp_repr_attr (object->repr, key);
+	str = sp_repr_attr (object->repr, key);
 
 	if (!strcmp (key, "x1")) {
-		lg->x1.distance = sp_svg_read_length (&lg->x1.unit, val, 0.0);
-		lg->x1_set = (val != NULL);
+		if (!sp_svg_length_read (str, &lg->x1)) {
+			sp_svg_length_unset (&lg->x1, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
+		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "y1")) {
-		lg->y1.distance = sp_svg_read_length (&lg->y1.unit, val, 0.0);
-		lg->y1_set = (val != NULL);
+		if (!sp_svg_length_read (str, &lg->y1)) {
+			sp_svg_length_unset (&lg->y1, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
+		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "x2")) {
-		lg->x2.distance = sp_svg_read_length (&lg->x2.unit, val, 1.0);
-		lg->x2_set = (val != NULL);
+		if (!sp_svg_length_read (str, &lg->x2)) {
+			sp_svg_length_unset (&lg->x2, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
+		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
 	} else if (!strcmp (key, "y2")) {
-		lg->y2.distance = sp_svg_read_length (&lg->y2.unit, val, 0.0);
-		lg->y2_set = (val != NULL);
+		if (!sp_svg_length_read (str, &lg->y2)) {
+			sp_svg_length_unset (&lg->y2, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
+		}
 		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		return;
+	} else {
+		if (SP_OBJECT_CLASS (lg_parent_class)->read_attr)
+			(* SP_OBJECT_CLASS (lg_parent_class)->read_attr) (object, key);
 	}
-
-	if (SP_OBJECT_CLASS (lg_parent_class)->read_attr)
-		(* SP_OBJECT_CLASS (lg_parent_class)->read_attr) (object, key);
 }
 
+/*
+ * Basically we have to deal with transformations
+ *
+ * 1) color2norm - maps point in (0,NCOLORS) vector to (0,1) vector
+ *    fixme: I do not know, how to deal with start > 0 and end < 1
+ * 2) norm2pos - maps (0,1) vector to x1,y1 - x2,y2
+ * 2) gradientTransform
+ * 3) bbox2user
+ * 4) ctm == userspace to pixel grid
+ */
+
 static SPPainter *
-sp_lineargradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opacity, ArtDRect *bbox)
+sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const ArtDRect *bbox)
 {
 	SPLinearGradient *lg;
 	SPGradient *gr;
 	SPLGPainter *lgp;
+	gdouble color2norm[6];
 
 	lg = SP_LINEARGRADIENT (ps);
 	gr = SP_GRADIENT (ps);
 
-	if (!gr->color) {
-		sp_gradient_ensure_colors (gr);
-	}
+	if (!gr->color) sp_gradient_ensure_colors (gr);
 
 	lgp = g_new (SPLGPainter, 1);
 
@@ -1130,58 +1152,94 @@ sp_lineargradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opaci
 
 	lgp->lg = lg;
 
-	lgp->opacity = (guint32) floor (opacity * 255.9999);
+	/* fixme: Technically speaking, we map NCOLORS on line [start,end] onto line [0,1] (Lauris) */
+	/* fixme: I almost think, we should fill color array start and end in that case (Lauris) */
+	/* fixme: The alternative would be to leave these just empty garbage or something similar (Lauris) */
+	/* fixme: Originally I had 1023.9999 here - not sure, whether we have really to cut out ceil int (Lauris) */
+	art_affine_scale (color2norm, gr->len / (gdouble) NCOLORS, gr->len / (gdouble) NCOLORS);
+	SP_PRINT_TRANSFORM ("color2norm", color2norm);
+	/* Now we have normalized vector */
 
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		gdouble b2c[6], vec2b[6], norm2vec[6], norm2c[6], c2norm[6];
-		/* fixme: we should use start & end here */
-		norm2vec[0] = gr->len / 1023.9999;
-		norm2vec[1] = 0.0;
-		norm2vec[2] = 0.0;
-		norm2vec[3] = gr->len / 1023.9999;
-		norm2vec[4] = 0.0;
-		norm2vec[5] = 0.0;
-#if 0
-		g_print ("\nnorm2vec: ");for (i = 0; i < 6; i++) g_print ("%g ", norm2vec[i]);g_print ("\n");
-#endif
-		/* fixme: gradient transform somewhere here */
-		vec2b[0] = lg->x2.distance - lg->x1.distance;
-		vec2b[1] = lg->y2.distance - lg->y1.distance;
-		vec2b[2] = lg->y2.distance - lg->y1.distance;
-		vec2b[3] = lg->x1.distance - lg->x2.distance;
-		vec2b[4] = lg->x1.distance;
-		vec2b[5] = lg->y1.distance;
-#if 0
-		g_print ("vec2b: ");for (i = 0; i < 6; i++) g_print ("%g ", vec2b[i]);g_print ("\n");
-#endif
-		b2c[0] = bbox->x1 - bbox->x0;
-		b2c[1] = 0.0;
-		b2c[2] = 0.0;
-		b2c[3] = bbox->y1 - bbox->y0;
-		b2c[4] = bbox->x0;
-		b2c[5] = bbox->y0;
-#if 0
-		g_print ("b2c: ");for (i = 0; i < 6; i++) g_print ("%g ", b2c[i]);g_print ("\n");
-#endif
-		art_affine_multiply (norm2c, norm2vec, vec2b);
-		art_affine_multiply (norm2c, norm2c, b2c);
-#if 0
-		g_print ("norm2c: ");for (i = 0; i < 6; i++) g_print ("%g ", norm2c[i]);g_print ("\n");
-#endif
-		art_affine_invert (c2norm, norm2c);
-#if 0
-		g_print ("c2norm: ");for (i = 0; i < 6; i++) g_print ("%g ", c2norm[i]);g_print ("\n");
-#endif
-		lgp->x0 = norm2c[4];
-		lgp->y0 = norm2c[5];
-		lgp->dx = c2norm[0];
-		lgp->dy = c2norm[2];
+		gdouble norm2pos[6], bbox2user[6];
+		gdouble color2pos[6], color2tpos[6], color2user[6], color2px[6], px2color[6];
+
+		/* This is easy case, as we can just ignore percenting here */
+		/* fixme: Still somewhat tricky, but I think I got it correct (lauris) */
+		norm2pos[0] = lg->x2.computed - lg->x1.computed;
+		norm2pos[1] = lg->y2.computed - lg->y1.computed;
+		norm2pos[2] = lg->y2.computed - lg->y1.computed;
+		norm2pos[3] = lg->x1.computed - lg->x2.computed;
+		norm2pos[4] = lg->x1.computed;
+		norm2pos[5] = lg->y1.computed;
+		SP_PRINT_TRANSFORM ("norm2pos", norm2pos);
+
+		/* gradientTransform goes here (Lauris) */
+		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
+
+		/* BBox to user coordinate system */
+		bbox2user[0] = bbox->x1 - bbox->x0;
+		bbox2user[1] = 0.0;
+		bbox2user[2] = 0.0;
+		bbox2user[3] = bbox->y1 - bbox->y0;
+		bbox2user[4] = bbox->x0;
+		bbox2user[5] = bbox->y0;
+		SP_PRINT_TRANSFORM ("bbox2user", bbox2user);
+
+		/* CTM goes here */
+		SP_PRINT_TRANSFORM ("ctm", ctm);
+
+		art_affine_multiply (color2pos, color2norm, norm2pos);
+		SP_PRINT_TRANSFORM ("color2pos", color2pos);
+		art_affine_multiply (color2tpos, color2pos, gr->transform);
+		SP_PRINT_TRANSFORM ("color2tpos", color2tpos);
+		art_affine_multiply (color2user, color2tpos, bbox2user);
+		SP_PRINT_TRANSFORM ("color2user", color2user);
+		art_affine_multiply (color2px, color2user, ctm);
+		SP_PRINT_TRANSFORM ("color2px", color2px);
+
+		art_affine_invert (px2color, color2px);
+		SP_PRINT_TRANSFORM ("px2color", px2color);
+
+		lgp->x0 = color2px[4];
+		lgp->y0 = color2px[5];
+		lgp->dx = px2color[0];
+		lgp->dy = px2color[2];
 	} else {
-		/* This is placeholder */
-		lgp->x0 = affine[4];
-		lgp->y0 = affine[5];
-		lgp->dx = 127.0 / (bbox->x1 - bbox->x0);
-		lgp->dy = 127.0 / (bbox->y1 - bbox->y0);
+		gdouble norm2pos[6];
+		gdouble color2pos[6], color2tpos[6], color2px[6], px2color[6];
+		/* Problem: What to do, if we have mixed lengths and percentages? */
+		/* Currently we do ignore percentages at all, but that is not good (lauris) */
+
+		/* fixme: Do percentages (Lauris) */
+		norm2pos[0] = lg->x2.computed - lg->x1.computed;
+		norm2pos[1] = lg->y2.computed - lg->y1.computed;
+		norm2pos[2] = lg->y2.computed - lg->y1.computed;
+		norm2pos[3] = lg->x1.computed - lg->x2.computed;
+		norm2pos[4] = lg->x1.computed;
+		norm2pos[5] = lg->y1.computed;
+		SP_PRINT_TRANSFORM ("norm2pos", norm2pos);
+
+		/* gradientTransform goes here (Lauris) */
+		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
+
+		/* CTM goes here */
+		SP_PRINT_TRANSFORM ("ctm", ctm);
+
+		art_affine_multiply (color2pos, color2norm, norm2pos);
+		SP_PRINT_TRANSFORM ("color2pos", color2pos);
+		art_affine_multiply (color2tpos, color2pos, gr->transform);
+		SP_PRINT_TRANSFORM ("color2tpos", color2tpos);
+		art_affine_multiply (color2px, color2tpos, ctm);
+		SP_PRINT_TRANSFORM ("color2px", color2px);
+
+		art_affine_invert (px2color, color2px);
+		SP_PRINT_TRANSFORM ("px2color", px2color);
+
+		lgp->x0 = color2px[4];
+		lgp->y0 = color2px[5];
+		lgp->dx = px2color[0];
+		lgp->dy = px2color[2];
 	}
 
 	return (SPPainter *) lgp;
@@ -1204,10 +1262,89 @@ sp_lineargradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set
 
 	lg = SP_LINEARGRADIENT (gr);
 
-	if (set_missing || lg->x1_set) sp_repr_set_double_attribute (repr, "x1", lg->x1.distance);
-	if (set_missing || lg->y1_set) sp_repr_set_double_attribute (repr, "y1", lg->y1.distance);
-	if (set_missing || lg->x2_set) sp_repr_set_double_attribute (repr, "x2", lg->x2.distance);
-	if (set_missing || lg->y2_set) sp_repr_set_double_attribute (repr, "y2", lg->y2.distance);
+	/* fixme: Write real units etc. (Lauris) */
+	if (set_missing || lg->x1.set) sp_repr_set_double_attribute (repr, "x1", lg->x1.computed);
+	if (set_missing || lg->y1.set) sp_repr_set_double_attribute (repr, "y1", lg->y1.computed);
+	if (set_missing || lg->x2.set) sp_repr_set_double_attribute (repr, "x2", lg->x2.computed);
+	if (set_missing || lg->y2.set) sp_repr_set_double_attribute (repr, "y2", lg->y2.computed);
+}
+
+void
+sp_lineargradient_from_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p)
+{
+	SPGradient *gr;
+	gdouble p2b[6];
+	gdouble x, y;
+
+	g_return_if_fail (lg != NULL);
+	g_return_if_fail (SP_IS_LINEARGRADIENT (lg));
+	g_return_if_fail (p != NULL);
+
+	gr = SP_GRADIENT (lg);
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		gdouble bbox2user[6];
+
+		/* BBox to user coordinate system */
+		bbox2user[0] = bbox->x1 - bbox->x0;
+		bbox2user[1] = 0.0;
+		bbox2user[2] = 0.0;
+		bbox2user[3] = bbox->y1 - bbox->y0;
+		bbox2user[4] = bbox->x0;
+		bbox2user[5] = bbox->y0;
+
+		/* CTM goes here */
+
+		art_affine_multiply (p2b, bbox2user, ctm);
+	} else {
+		memcpy (p2b, ctm, 6 * sizeof (gdouble));
+	}
+
+	x = p2b[0] * p->x + p2b[2] * p->y + p2b[4];
+	y = p2b[1] * p->x + p2b[3] * p->y + p2b[5];
+
+	p->x = x;
+	p->y = y;
+}
+
+void
+sp_lineargradient_to_position (SPLinearGradient *lg, gdouble *ctm, ArtDRect *bbox, ArtPoint *p)
+{
+	SPGradient *gr;
+	gdouble p2b[6], b2p[6];
+	gdouble x, y;
+
+	g_return_if_fail (lg != NULL);
+	g_return_if_fail (SP_IS_LINEARGRADIENT (lg));
+	g_return_if_fail (p != NULL);
+
+	gr = SP_GRADIENT (lg);
+
+	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+		gdouble bbox2user[6];
+
+		/* BBox to user coordinate system */
+		bbox2user[0] = bbox->x1 - bbox->x0;
+		bbox2user[1] = 0.0;
+		bbox2user[2] = 0.0;
+		bbox2user[3] = bbox->y1 - bbox->y0;
+		bbox2user[4] = bbox->x0;
+		bbox2user[5] = bbox->y0;
+
+		/* CTM goes here */
+
+		art_affine_multiply (p2b, bbox2user, ctm);
+
+		art_affine_invert (b2p, p2b);
+	} else {
+		art_affine_invert (b2p, ctm);
+	}
+
+	x = b2p[0] * p->x + b2p[2] * p->y + b2p[4];
+	y = b2p[1] * p->x + b2p[3] * p->y + b2p[5];
+
+	p->x = x;
+	p->y = y;
 }
 
 void
@@ -1216,11 +1353,11 @@ sp_lineargradient_set_position (SPLinearGradient *lg, gdouble x1, gdouble y1, gd
 	g_return_if_fail (lg != NULL);
 	g_return_if_fail (SP_IS_LINEARGRADIENT (lg));
 
-	/* fixme: units */
-	lg->x1.distance = x1;
-	lg->y1.distance = y1;
-	lg->x2.distance = x2;
-	lg->y2.distance = y2;
+	/* fixme: units? (Lauris)  */
+	sp_svg_length_unset (&lg->x1, SP_SVG_UNIT_NONE, x1, x1);
+	sp_svg_length_unset (&lg->y1, SP_SVG_UNIT_NONE, y1, y1);
+	sp_svg_length_unset (&lg->x2, SP_SVG_UNIT_NONE, x2, x2);
+	sp_svg_length_unset (&lg->y2, SP_SVG_UNIT_NONE, y2, y2);
 
 	sp_object_request_modified (SP_OBJECT (lg), SP_OBJECT_MODIFIED_FLAG);
 }
@@ -1256,46 +1393,39 @@ sp_lg_fill (SPPainter *painter, guchar *px, gint x0, gint y0, gint width, gint h
 	gdouble pos;
 	gint x, y;
 	guchar *p;
-	guint tmp;
 
 	lgp = (SPLGPainter *) painter;
 	lg = lgp->lg;
 	g = (SPGradient *) lg;
 
 	if (!g->color) {
+#if 0
 		/* fixme: This is forbidden, so we should paint some mishmesh instead */
 		sp_gradient_ensure_colors (g);
+#else
+		nr_render_r8g8b8a8_gray_garbage (px, width, height, rowstride);
+		return;
+#endif
 	}
 
 	for (y = 0; y < height; y++) {
 		p = px + y * rowstride;
 		pos = (y + y0 - lgp->y0) * lgp->dy + (0 + x0 - lgp->x0) * lgp->dx;
 		for (x = 0; x < width; x++) {
-			gint ip, idx;
-			ip = (gint) pos;
-			switch (g->spread) {
-			case SP_GRADIENT_SPREAD_PAD:
-				idx = CLAMP (ip, 0, 1023);
-				break;
-			case SP_GRADIENT_SPREAD_REFLECT:
-				idx = ip & 0x3ff;
-				if (ip & 0x400) idx = 1023 - idx;
-				break;
-			case SP_GRADIENT_SPREAD_REPEAT:
-				idx = ip & 0x3ff;
-				break;
-			default:
-				g_assert_not_reached ();
-				idx = 0;
-				break;
+			gint idx;
+			if (g->spread == SP_GRADIENT_SPREAD_REFLECT) {
+				idx = ((gint) pos) & (2 * NCOLORS - 1);
+				if (idx & NCOLORS) idx = (2 * NCOLORS) - idx;
+			} else if (g->spread == SP_GRADIENT_SPREAD_REPEAT) {
+				idx = ((gint) pos) & (NCOLORS - 1);
+			} else {
+				idx = CLAMP (((gint) pos), 0, (NCOLORS - 1));
 			}
-			/* a * b / 255 = a * b * (256 * 256 / 255) / 256 / 256 */
-			tmp = g->color[4 * idx + 3] * lgp->opacity;
-			tmp = (tmp << 16) + tmp + (tmp >> 8);
-			*p++ = g->color[4 * idx];
-			*p++ = g->color[4 * idx + 1];
-			*p++ = g->color[4 * idx + 2];
-			*p++ = tmp >> 24;
+			idx = idx * 4;
+			*p++ = g->color[idx];
+			*p++ = g->color[idx + 1];
+			*p++ = g->color[idx + 2];
+			*p++ = g->color[idx + 3];
 			pos += lgp->dx;
 		}
 	}
@@ -1323,7 +1453,7 @@ static void sp_radialgradient_destroy (GtkObject *object);
 static void sp_radialgradient_build (SPObject *object, SPDocument *document, SPRepr *repr);
 static void sp_radialgradient_read_attr (SPObject *object, const gchar *key);
 
-static SPPainter *sp_radialgradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opacity, ArtDRect *bbox);
+static SPPainter *sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox);
 static void sp_radialgradient_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static void sp_radialgradient_flatten_attributes (SPGradient *gradient, SPRepr *repr, gboolean set_missing);
@@ -1379,16 +1509,11 @@ sp_radialgradient_class_init (SPRadialGradientClass * klass)
 static void
 sp_radialgradient_init (SPRadialGradient *rg)
 {
-	rg->cx.distance = 0.5;
-	rg->cx.unit = sp_unit_get_by_abbreviation ("%");
-	rg->cy.distance = 0.5;
-	rg->cy.unit = sp_unit_get_by_abbreviation ("%");
-	rg->r.distance = 0.5;
-	rg->r.unit = sp_unit_get_by_abbreviation ("%");
-	rg->fx.distance = 0.5;
-	rg->fx.unit = sp_unit_get_by_abbreviation ("%");
-	rg->fy.distance = 0.5;
-	rg->fy.unit = sp_unit_get_by_abbreviation ("%");
+	sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+	sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+	sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+	sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+	sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
 }
 
 static void
@@ -1423,40 +1548,45 @@ static void
 sp_radialgradient_read_attr (SPObject *object, const gchar *key)
 {
 	SPRadialGradient *rg;
-	const gchar *val;
+	const guchar *str;
 
 	rg = SP_RADIALGRADIENT (object);
 
-	val = sp_repr_attr (object->repr, key);
+	str = sp_repr_attr (object->repr, key);
 
 	if (!strcmp (key, "cx")) {
-		rg->cx.distance = sp_svg_read_length (&rg->cx.unit, val, 0.0);
-		rg->cx_set = (val != NULL);
-		return;
+		if (!sp_svg_length_read (str, &rg->cx)) {
+			sp_svg_length_unset (&rg->cx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "cy")) {
-		rg->cy.distance = sp_svg_read_length (&rg->cy.unit, val, 0.0);
-		rg->cy_set = (val != NULL);
-		return;
+		if (!sp_svg_length_read (str, &rg->cy)) {
+			sp_svg_length_unset (&rg->cy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "r")) {
-		rg->r.distance = sp_svg_read_length (&rg->r.unit, val, 0.0);
-		rg->r_set = (val != NULL);
-		return;
+		if (!sp_svg_length_read (str, &rg->r)) {
+			sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "fx")) {
-		rg->fx.distance = sp_svg_read_length (&rg->fx.unit, val, 1.0);
-		rg->fx_set = (val != NULL);
-		return;
+		if (!sp_svg_length_read (str, &rg->fx)) {
+			sp_svg_length_unset (&rg->fx, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
 	} else if (!strcmp (key, "fy")) {
-		rg->fy.distance = sp_svg_read_length (&rg->fy.unit, val, 0.0);
-		rg->fy_set = (val != NULL);
-		return;
+		if (!sp_svg_length_read (str, &rg->fy)) {
+			sp_svg_length_unset (&rg->fy, SP_SVG_UNIT_PERCENT, 0.5, 50.0);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+	} else {
+		if (SP_OBJECT_CLASS (rg_parent_class)->read_attr)
+			(* SP_OBJECT_CLASS (rg_parent_class)->read_attr) (object, key);
 	}
-
-	if (SP_OBJECT_CLASS (rg_parent_class)->read_attr)
-		(* SP_OBJECT_CLASS (rg_parent_class)->read_attr) (object, key);
 }
 
 static SPPainter *
-sp_radialgradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opacity, ArtDRect *bbox)
+sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const ArtDRect *bbox)
 {
 	SPRadialGradient *rg;
 	SPGradient *gr;
@@ -1476,8 +1606,6 @@ sp_radialgradient_painter_new (SPPaintServer *ps, gdouble *affine, gdouble opaci
 
 	rgp->rg = rg;
 
-	rgp->opacity = (guint) floor (opacity * 255.9999);
-
 #if 0
 	if (rg->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
 		gdouble b2c[6], vec2b[6], norm2vec[6], norm2c[6], c2norm[6];
@@ -1551,11 +1679,11 @@ sp_radialgradient_flatten_attributes (SPGradient *gr, SPRepr *repr, gboolean set
 
 	rg = SP_RADIALGRADIENT (gr);
 
-	if (set_missing || rg->cx_set) sp_repr_set_double_attribute (repr, "cx", rg->cx.distance);
-	if (set_missing || rg->cy_set) sp_repr_set_double_attribute (repr, "cy", rg->cy.distance);
-	if (set_missing || rg->r_set) sp_repr_set_double_attribute (repr, "r", rg->r.distance);
-	if (set_missing || rg->fx_set) sp_repr_set_double_attribute (repr, "fx", rg->fx.distance);
-	if (set_missing || rg->fy_set) sp_repr_set_double_attribute (repr, "fy", rg->fy.distance);
+	if (set_missing || rg->cx.set) sp_repr_set_double_attribute (repr, "cx", rg->cx.computed);
+	if (set_missing || rg->cy.set) sp_repr_set_double_attribute (repr, "cy", rg->cy.computed);
+	if (set_missing || rg->r.set) sp_repr_set_double_attribute (repr, "r", rg->r.computed);
+	if (set_missing || rg->fx.set) sp_repr_set_double_attribute (repr, "fx", rg->fx.computed);
+	if (set_missing || rg->fy.set) sp_repr_set_double_attribute (repr, "fy", rg->fy.computed);
 }
 
 /* Builds flattened repr tree of gradient - i.e. no href */
