@@ -1,15 +1,15 @@
 #define __SP_GROUP_C__
 
 /*
- * SPItemGroup
+ * SVG <g> implementation
  *
- * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2000 Lauris Kaplinski
+ * Copyright (C) 1999-2002 authors
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
- * Released under GNU GPL
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <config.h>
@@ -23,6 +23,7 @@
 #include "sp-object-repr.h"
 #include "svg/svg.h"
 #include "document.h"
+#include "style.h"
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "selection.h"
@@ -37,12 +38,10 @@ static void sp_group_read_attr (SPObject * object, const gchar * attr);
 static void sp_group_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
 static void sp_group_remove_child (SPObject * object, SPRepr * child);
 static void sp_group_order_changed (SPObject * object, SPRepr * child, SPRepr * old, SPRepr * new);
-static void sp_group_style_changed (SPObject *object, guint flags);
 static void sp_group_modified (SPObject *object, guint flags);
 static gint sp_group_sequence (SPObject *object, gint seq);
 
-static void sp_group_update (SPItem *item, gdouble affine[]);
-static void sp_group_bbox (SPItem * item, ArtDRect * bbox);
+static void sp_group_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform);
 static void sp_group_print (SPItem * item, GnomePrintContext * gpc);
 static gchar * sp_group_description (SPItem * item);
 static NRArenaItem *sp_group_show (SPItem *item, NRArena *arena);
@@ -91,11 +90,9 @@ sp_group_class_init (SPGroupClass *klass)
 	sp_object_class->child_added = sp_group_child_added;
 	sp_object_class->remove_child = sp_group_remove_child;
 	sp_object_class->order_changed = sp_group_order_changed;
-	sp_object_class->style_changed = sp_group_style_changed;
 	sp_object_class->modified = sp_group_modified;
 	sp_object_class->sequence = sp_group_sequence;
 
-	item_class->update = sp_group_update;
 	item_class->bbox = sp_group_bbox;
 	item_class->print = sp_group_print;
 	item_class->description = sp_group_description;
@@ -161,10 +158,10 @@ sp_group_read_attr (SPObject * object, const gchar * attr)
 }
 
 static void
-sp_group_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
+sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 {
-	SPGroup * group;
-	SPItem * item;
+	SPGroup *group;
+	SPItem *item;
 	GtkType type;
 	SPObject *ochild, *prev;
 	gint position;
@@ -180,7 +177,7 @@ sp_group_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 	position = 0;
 	if (ref != NULL) {
 		prev = group->children;
-		while (prev->repr != ref) {
+		while (prev && (prev->repr != ref)) {
 			if (SP_IS_ITEM (prev)) position += 1;
 			prev = prev->next;
 		}
@@ -241,77 +238,61 @@ sp_group_remove_child (SPObject * object, SPRepr * child)
 }
 
 static void
-sp_group_order_changed (SPObject * object, SPRepr * child, SPRepr * old, SPRepr * new)
+sp_group_order_changed (SPObject *object, SPRepr *child, SPRepr *old, SPRepr *new)
 {
-	SPGroup * group;
-	SPObject * ochild, * oold, * onew, * o;
-	gint pos, oldpos, newpos;
+	SPGroup *group;
+	SPObject *childobj, *oldobj, *newobj, *o;
+	gint childpos, oldpos, newpos;
 
 	group = SP_GROUP (object);
 
 	if (((SPObjectClass *) (parent_class))->order_changed)
 		(* ((SPObjectClass *) (parent_class))->order_changed) (object, child, old, new);
 
-	ochild = NULL;
-	oold = NULL;
-	onew = NULL;
-	pos = oldpos = newpos = 0;
+	childobj = oldobj = newobj = NULL;
+	oldpos = newpos = 0;
 
-	o = group->children;
-	while ((!ochild) || (old && !oold) || (new && !onew)) {
-		if (SP_IS_ITEM (o)) pos += 1;
-		if (o->repr == child) ochild = o;
+	/* Scan children list */
+	childpos = 0;
+	for (o = group->children; !childobj || (old && !oldobj) || (new && !newobj); o = o->next) {
+		if (o->repr == child) {
+			childobj = o;
+		} else {
+			if (SP_IS_ITEM (o)) childpos += 1;
+		}
 		if (old && o->repr == old) {
-			oold = o;
-			oldpos = pos;
+			oldobj = o;
+			oldpos = childpos;
 		}
 		if (new && o->repr == new) {
-			onew = o;
-			newpos = pos;
+			newobj = o;
+			newpos = childpos;
 		}
-		o = o->next;
 	}
 
 	g_print ("oldpos %d newpos %d\n", oldpos, newpos);
 
-	if (oold) {
-		oold->next = ochild->next;
+	if (oldobj) {
+		oldobj->next = childobj->next;
 	} else {
-		group->children = ochild->next;
+		group->children = childobj->next;
 	}
-	if (onew) {
-		ochild->next = onew->next;
-		onew->next = ochild;
+	if (newobj) {
+		childobj->next = newobj->next;
+		newobj->next = childobj;
 	} else {
-		ochild->next = group->children;
-		group->children = ochild;
+		childobj->next = group->children;
+		group->children = childobj;
 	}
 
-	if (SP_IS_ITEM (ochild)) {
+	if (SP_IS_ITEM (childobj)) {
 		SPItemView *v;
-		for (v = SP_ITEM (ochild)->display; v != NULL; v = v->next) {
+		for (v = SP_ITEM (childobj)->display; v != NULL; v = v->next) {
 			nr_arena_item_set_order (v->arenaitem, newpos);
 		}
 	}
 }
 
-static void
-sp_group_style_changed (SPObject *object, guint flags)
-{
-	SPGroup *group;
-	SPObject *child;
-
-	group = SP_GROUP (object);
-
-	if (((SPObjectClass *) (parent_class))->style_changed)
-		(* ((SPObjectClass *) (parent_class))->style_changed) (object, flags);
-
-	/* fixme: I am not sure - currently we are invoking 2 cascades - style changed and mofified */
-	for (child = group->children; child != NULL; child = child->next) {
-		sp_object_style_changed (child, SP_OBJECT_PARENT_MODIFIED_FLAG);
-	}
-}
-
 static void
 sp_group_modified (SPObject *object, guint flags)
 {
@@ -322,7 +303,7 @@ sp_group_modified (SPObject *object, guint flags)
 	group = SP_GROUP (object);
 
 	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
-	flags &= SP_OBJECT_PARENT_MODIFIED_FLAG;
+	flags &= SP_OBJECT_MODIFIED_CASCADE;
 
 	l = NULL;
 	for (child = group->children; child != NULL; child = child->next) {
@@ -358,29 +339,7 @@ sp_group_sequence (SPObject *object, gint seq)
 }
 
 static void
-sp_group_update (SPItem *item, gdouble affine[])
-{
-	SPGroup *group;
-	SPItem * child;
-	SPObject * o;
-	gdouble a[6];
-
-	group = SP_GROUP (item);
-
-	if (SP_ITEM_CLASS (parent_class)->update)
-		(* SP_ITEM_CLASS (parent_class)->update) (item, affine);
-
-	for (o = group->children; o != NULL; o = o->next) {
-		if (SP_IS_ITEM (o)) {
-			child = SP_ITEM (o);
-			art_affine_multiply (a, child->affine, affine);
-			sp_item_update (child, a);
-		}
-	}
-}
-
-static void
-sp_group_bbox (SPItem * item, ArtDRect *bbox)
+sp_group_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 {
 	SPGroup * group;
 	SPItem * child;
@@ -393,8 +352,10 @@ sp_group_bbox (SPItem * item, ArtDRect *bbox)
 
 	for (o = group->children; o != NULL; o = o->next) {
 		if (SP_IS_ITEM (o)) {
+			gdouble a[6];
 			child = SP_ITEM (o);
-			sp_item_bbox (child, &child_bbox);
+			art_affine_multiply (a, child->affine, transform);
+			sp_item_invoke_bbox (child, &child_bbox, a);
 			art_drect_union (bbox, bbox, &child_bbox);
 		}
 	}
@@ -547,64 +508,68 @@ sp_item_group_ungroup_activate (GtkMenuItem *menuitem, SPGroup *group)
 void
 sp_item_group_ungroup (SPGroup *group, GSList **children)
 {
-	SPDocument *document;
-	SPItem *gitem;
-	SPRepr *grepr;
-	gdouble gtrans[6];
+	SPDocument *doc;
+	SPItem *gitem, *pitem;
+	SPRepr *grepr, *prepr, *lrepr;
 
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (SP_IS_GROUP (group));
 
-	document = ((SPObject *) group)->document;
+	doc = SP_OBJECT_DOCUMENT (group);
 
 	gitem = SP_ITEM (group);
+	grepr = SP_OBJECT_REPR (gitem);
+	pitem = SP_ITEM (SP_OBJECT_PARENT (gitem));
+	prepr = SP_OBJECT_REPR (pitem);
 
-	grepr = ((SPObject *) gitem)->repr;
 	g_return_if_fail (!strcmp (sp_repr_name (grepr), "g") || !strcmp (sp_repr_name (grepr), "a"));
-	if (gitem->affine) {
-		memcpy (gtrans, gitem->affine, 6 * sizeof (gdouble));
-	} else {
-		art_affine_identity (gtrans);
-	}
+
+	lrepr = grepr;
 
 	while (group->children) {
-		SPRepr *crepr;
-
-		crepr = group->children->repr;
-
 		if (SP_IS_ITEM (group->children)) {
-			SPRepr *nrepr;
-			SPItem *nitem;
+			SPRepr *crepr, *nrepr;
+			SPItem *citem, *nitem;
 			gdouble ctrans[6];
-			const gchar *castr;
-			SPCSSAttr * css;
 			gchar affinestr[80];
+			guchar *ss;
+
+			crepr = SP_OBJECT_REPR (group->children);
+			citem = SP_ITEM (group->children);
 
 			nrepr = sp_repr_duplicate (crepr);
 
-			art_affine_identity (ctrans);
-			castr = sp_repr_attr (nrepr, "transform");
-			sp_svg_read_affine (ctrans, castr);
-			art_affine_multiply (ctrans, ctrans, gtrans);
-			sp_svg_write_affine (affinestr, 79, ctrans);
-			affinestr[79] = '\0';
-			sp_repr_set_attr (nrepr, "transform", affinestr);
-
-			css = sp_repr_css_attr_inherited (nrepr, "style");
-			sp_repr_css_set (nrepr, css, "style");
-			sp_repr_css_attr_unref (css);
+			art_affine_multiply (ctrans, citem->affine, gitem->affine);
+			if (sp_svg_write_affine (affinestr, 79, ctrans)) {
+				sp_repr_set_attr (nrepr, "transform", affinestr);
+			} else {
+				sp_repr_set_attr (nrepr, "transform", NULL);
+			}
 
+			/* Merging of style */
+			/* fixme: We really should respect presentation attributes too */
+			ss = sp_style_write_difference (SP_OBJECT_STYLE (citem), SP_OBJECT_STYLE (pitem));
+			sp_repr_set_attr (nrepr, "style", ss);
+			g_free (ss);
+#if 0
 			/* fixme: Sort up that item/object stuff (lauris) */
 			nitem = (SPItem *) sp_document_add_repr (document, nrepr);
 			sp_repr_unref (nrepr);
 			if (children && SP_IS_ITEM (nitem)) *children = g_slist_prepend (*children, nitem);
+#else
+			/* Append new children after group */
+			sp_repr_add_child (prepr, nrepr, lrepr);
+			lrepr = nrepr;
+			nitem = (SPItem *) sp_document_lookup_id (doc, sp_repr_attr (nrepr, "id"));
+			sp_repr_unref (nrepr);
+			if (children && SP_IS_ITEM (nitem)) *children = g_slist_prepend (*children, nitem);
+#endif
 		}
-
-		sp_repr_remove_child (grepr, crepr);
+		sp_repr_remove_child (grepr, SP_OBJECT_REPR (group->children));
 	}
 
 	sp_repr_unparent (grepr);
-	sp_document_done (document);
+	sp_document_done (doc);
 }
 
 /*
@@ -614,16 +579,17 @@ sp_item_group_ungroup (SPGroup *group, GSList **children)
 GSList * 
 sp_item_group_item_list (SPGroup * group)
 {
-        GSList * s;
-	SPObject * o;
+        GSList *s;
+	SPObject *o;
 
 	g_return_val_if_fail (group != NULL, NULL);
 	g_return_val_if_fail (SP_IS_GROUP (group), NULL);
 
 	s = NULL;
 
-	for (o = group->children; o != NULL; o = o->next) 
-		if (SP_IS_ITEM (o)) s = g_slist_append (s, o);
+	for (o = group->children; o != NULL; o = o->next) {
+		if (SP_IS_ITEM (o)) s = g_slist_prepend (s, o);
+	}
 
-	return s;
+	return g_slist_reverse (s);
 }
