@@ -1,4 +1,16 @@
-#define SP_ITEM_C
+#define __SP_ITEM_C__
+
+/*
+ * Base class for visual SVG elements
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 2001-2002 Lauris Kaplinski
+ * Copyright (C) 2001 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <config.h>
 #include <math.h>
@@ -6,6 +18,8 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include "helper/art-utils.h"
+#include "helper/nr-plain-stuff.h"
 #include "svg/svg.h"
 #include "display/nr-arena.h"
 #include "display/nr-arena-item.h"
@@ -19,17 +33,21 @@
 /* fixme: I do not like that (Lauris) */
 #include "dialogs/item-properties.h"
 #include "dialogs/object-attributes.h"
+#include "sp-root.h"
 #include "sp-anchor.h"
 #include "sp-clippath.h"
 #include "sp-item.h"
 
+#define noSP_ITEM_DEBUG_IDLE
+
 static void sp_item_class_init (SPItemClass * klass);
 static void sp_item_init (SPItem * item);
 static void sp_item_destroy (GtkObject * object);
 
 static void sp_item_build (SPObject * object, SPDocument * document, SPRepr * repr);
 static void sp_item_read_attr (SPObject *object, const gchar *key);
-static void sp_item_style_changed (SPObject *object, guint flags);
+static void sp_item_modified (SPObject *object, guint flags);
+static void sp_item_style_modified (SPObject *object, guint flags);
 
 static gchar * sp_item_private_description (SPItem * item);
 static GSList * sp_item_private_snappoints (SPItem * item, GSList * points);
@@ -81,7 +99,8 @@ sp_item_class_init (SPItemClass * klass)
 
 	sp_object_class->build = sp_item_build;
 	sp_object_class->read_attr = sp_item_read_attr;
-	sp_object_class->style_changed = sp_item_style_changed;
+	sp_object_class->modified = sp_item_modified;
+	sp_object_class->style_modified = sp_item_style_modified;
 
 	klass->description = sp_item_private_description;
 	klass->show = sp_item_private_show;
@@ -105,7 +124,7 @@ sp_item_init (SPItem *item)
 
 	item->clip = NULL;
 
-	if (!object->style) object->style = sp_style_new (SP_OBJECT (item));
+	if (!object->style) object->style = sp_style_new_from_object (SP_OBJECT (item));
 }
 
 static void
@@ -168,8 +187,6 @@ sp_item_read_attr (SPObject * object, const gchar * key)
 		art_affine_identity (a);
 		if (astr != NULL) sp_svg_read_affine (a, astr);
 		sp_item_set_item_transform (item, a);
-		/* fixme: in update */
-		object->style->real_stroke_width_set = FALSE;
 	}
 
 	if (!strcmp (key, "clip-path")) {
@@ -208,17 +225,45 @@ sp_item_read_attr (SPObject * object, const gchar * key)
 
 	/* fixme: */
 	if (!strcmp (key, "style") ||
+	    !strcmp (key, "font-size") ||
 	    !strcmp (key, "fill-cmyk") ||
 	    !strcmp (key, "fill") ||
+	    !strcmp (key, "stroke-cmyk") ||
 	    !strcmp (key, "stroke") ||
 	    !strcmp (key, "opacity")) {
 		sp_style_read_from_object (object->style, object);
-		sp_object_style_changed (object, SP_OBJECT_MODIFIED_FLAG);
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 	}
 }
 
 static void
-sp_item_style_changed (SPObject *object, guint flags)
+sp_item_modified (SPObject *object, guint flags)
+{
+	SPItem *item;
+	SPStyle *style;
+
+	item = SP_ITEM (object);
+	style = SP_OBJECT_STYLE (object);
+
+#ifdef SP_ITEM_DEBUG_IDLE
+	g_print ("M");
+#endif
+
+	if (style->stroke_width.unit == SP_CSS_UNIT_PERCENT) {
+		gdouble i2vp[6], vp2i[6];
+		gdouble aw, ah;
+		/* fixme: It is somewhat dangerous, yes (lauris) */
+		sp_item_i2vp_affine (item, i2vp);
+		art_affine_invert (vp2i, i2vp);
+		aw = sp_distance_d_matrix_d_transform (1.0, vp2i);
+		ah = sp_distance_d_matrix_d_transform (1.0, vp2i);
+		/* sqrt ((actual_width) ** 2 + (actual_height) ** 2)) / sqrt (2) */
+		style->stroke_width.computed = style->stroke_width.value * sqrt (aw * aw + ah * ah) * M_SQRT1_2;
+	}
+}
+
+static void
+sp_item_style_modified (SPObject *object, guint flags)
 {
 	SPItem *item;
 	SPStyle *style;
@@ -229,6 +274,11 @@ sp_item_style_changed (SPObject *object, guint flags)
 
 	/* Set up inherited/relative style properties */
 
+#ifdef SP_ITEM_DEBUG_IDLE
+	g_print ("S");
+#endif
+
+#if 0
 	if (!style->real_opacity_set) {
 		SPObject *parent;
 		style->real_opacity = style->opacity;
@@ -238,55 +288,44 @@ sp_item_style_changed (SPObject *object, guint flags)
 		if (parent) style->real_opacity = style->real_opacity * parent->style->real_opacity;
 		style->real_opacity_set = TRUE;
 	}
+#endif
 
-	if (!style->real_stroke_width_set) {
-		gdouble i2doc[6], dx, dy;
-		gdouble a2u, u2a;
-		sp_item_i2doc_affine (item, i2doc);
-		dx = i2doc[0] + i2doc[2];
-		dy = i2doc[1] + i2doc[3];
-		u2a = sqrt (dx * dx + dy * dy) * 0.707106781;
-		a2u = u2a > 1e-9 ? 1 / u2a : 1e9;
-		/* Calculate actual stroke width */
-		style->absolute_stroke_width = sp_item_distance_to_svg_viewport (item, style->stroke_width.distance, style->stroke_width.unit);
-		style->user_stroke_width = style->absolute_stroke_width * a2u;
-		style->real_stroke_width_set = TRUE;
-	}
 
 	for (v = item->display; v != NULL; v = v->next) {
-		nr_arena_item_set_opacity (v->arenaitem, object->style->opacity);
+		nr_arena_item_set_opacity (v->arenaitem, SP_SCALE24_TO_FLOAT (object->style->opacity.value));
 	}
 }
 
-/* Update indicates that affine is changed */
-
 void
-sp_item_update (SPItem * item, gdouble affine[])
-{
-	g_assert (item != NULL);
-	g_assert (SP_IS_ITEM (item));
-	g_assert (affine != NULL);
-
-	if (SP_ITEM_CLASS (((GtkObject *)(item))->klass)->update)
-		(* SP_ITEM_CLASS (((GtkObject *)(item))->klass)->update) (item, affine);
-}
-
-void sp_item_bbox (SPItem * item, ArtDRect * bbox)
+sp_item_invoke_bbox (SPItem *item, ArtDRect *bbox, const gdouble *transform)
 {
 	g_assert (item != NULL);
 	g_assert (SP_IS_ITEM (item));
 	g_assert (bbox != NULL);
+	g_assert (transform != NULL);
 
-	bbox->x0 = bbox->y0 = 1.0;
+	bbox->x0 = bbox->y0 = 0.0;
 	bbox->x1 = bbox->y1 = 0.0;
 
 	if (SP_ITEM_CLASS (((GtkObject *)(item))->klass)->bbox)
-		(* SP_ITEM_CLASS (((GtkObject *)(item))->klass)->bbox) (item, bbox);
+		SP_ITEM_CLASS (((GtkObject *)(item))->klass)->bbox (item, bbox, transform);
+}
+
+void sp_item_bbox_desktop (SPItem *item, ArtDRect *bbox)
+{
+	gdouble i2d[6];
+
+	g_assert (item != NULL);
+	g_assert (SP_IS_ITEM (item));
+	g_assert (bbox != NULL);
+
+	sp_item_i2d_affine (item, i2d);
+
+	sp_item_invoke_bbox (item, bbox, i2d);
 }
 
 SPKnotHolder *
-sp_item_knot_holder (SPItem    *item,
-	       SPDesktop *desktop)
+sp_item_knot_holder (SPItem *item, SPDesktop *desktop)
 {
 	SPKnotHolder *knot_holder = NULL;
 
@@ -304,11 +343,13 @@ sp_item_private_snappoints (SPItem * item, GSList * points)
 {
         ArtDRect bbox;
 	ArtPoint * p;
+	gdouble i2d[6];
 
 	g_assert (item != NULL);
 	g_assert (SP_IS_ITEM (item));
 
-	sp_item_bbox (item, &bbox);
+	sp_item_i2d_affine (item, i2d);
+	sp_item_invoke_bbox (item, &bbox, i2d);
 
 	p = g_new (ArtPoint,1);
 	p->x = bbox.x0;
@@ -396,7 +437,7 @@ sp_item_show (SPItem *item, NRArena *arena)
 	if (ai != NULL) {
 		item->display = sp_item_view_new_prepend (item->display, item, arena, ai);
 		nr_arena_item_set_transform (ai, item->affine);
-		nr_arena_item_set_opacity (ai, SP_OBJECT_STYLE (item)->opacity);
+		nr_arena_item_set_opacity (ai, SP_SCALE24_TO_FLOAT (SP_OBJECT_STYLE (item)->opacity.value));
 		nr_arena_item_set_sensitive (ai, item->sensitive);
 		if (item->clip) {
 			NRArenaItem *ac;
@@ -404,11 +445,7 @@ sp_item_show (SPItem *item, NRArena *arena)
 			nr_arena_item_set_clip (ai, ac);
 			nr_arena_item_unref (ac);
 		}
-#if 0
-		sp_desktop_connect_item (desktop, item, canvasitem);
-#else
 		gtk_object_set_user_data (GTK_OBJECT (ai), item);
-#endif
 	}
 
 	return ai;
@@ -450,13 +487,14 @@ sp_item_paint (SPItem *item, ArtPixBuf *buf, gdouble affine[])
 	NRIRect bbox;
 	NRGC gc;
 	NRBuffer *b;
-	gint y;
 
 	g_assert (item != NULL);
 	g_assert (SP_IS_ITEM (item));
 	g_assert (buf != NULL);
 	g_assert (affine != NULL);
 
+	sp_document_ensure_up_to_date (SP_OBJECT_DOCUMENT (item));
+
 	/* Create new arena */
 	arena = gtk_type_new (NR_TYPE_ARENA);
 	/* Create ArenaItem and set transform */
@@ -478,9 +516,7 @@ sp_item_paint (SPItem *item, ArtPixBuf *buf, gdouble affine[])
 	sp_item_hide (item, arena);
 	gtk_object_unref (GTK_OBJECT (arena));
 	/* Copy buffer to output */
-	for (y = 0; y < buf->height; y++) {
-		memcpy (buf->pixels + y * buf->rowstride, b->px + y * b->rs, 4 * buf->width);
-	}
+	nr_render_r8g8b8a8_r8g8b8a8_alpha (buf->pixels, buf->width, buf->height, buf->rowstride, b->px, b->rs, 255);
 	/* Release RGBA buffer */
 	nr_buffer_free (b);
 
@@ -503,8 +539,11 @@ sp_item_write_transform (SPItem *item, SPRepr *repr, gdouble *transform)
 			((SPItemClass *) (((GtkObject *) item)->klass))->write_transform (item, repr, ltrans);
 		} else {
 			guchar t[80];
-			sp_svg_write_affine (t, 80, item->affine);
-			sp_repr_set_attr (SP_OBJECT_REPR (item), "transform", t);
+			if (sp_svg_write_affine (t, 80, item->affine)) {
+				sp_repr_set_attr (SP_OBJECT_REPR (item), "transform", t);
+			} else {
+				sp_repr_set_attr (SP_OBJECT_REPR (item), "transform", t);
+			}
 		}
 	}
 }
@@ -565,6 +604,41 @@ sp_item_i2doc_affine (SPItem * item, gdouble affine[])
 	return affine;
 }
 
+/* Transformation to normalized (0,0-1,1) viewport */
+
+gdouble *
+sp_item_i2vp_affine (SPItem *item, gdouble affine[])
+{
+	SPRoot *root;
+
+	g_return_val_if_fail (item != NULL, NULL);
+	g_return_val_if_fail (SP_IS_ITEM (item), NULL);
+	g_return_val_if_fail (affine != NULL, NULL);
+
+	art_affine_identity (affine);
+
+	while (SP_OBJECT_PARENT (item)) {
+		art_affine_multiply (affine, affine, item->affine);
+		item = (SPItem *) SP_OBJECT_PARENT (item);
+	}
+
+	g_return_val_if_fail (SP_IS_ROOT (item), NULL);
+
+	/* fixme: Viewbox is specified using root transformation (Lauris) */
+	art_affine_multiply (affine, affine, item->affine);
+
+	root = SP_ROOT (item);
+
+	affine[0] /= root->width.computed;
+	affine[1] /= root->height.computed;
+	affine[2] /= root->width.computed;
+	affine[3] /= root->height.computed;
+
+	return affine;
+}
+
+/* fixme: This is EVIL!!! */
+
 gdouble *
 sp_item_i2d_affine (SPItem *item, gdouble affine[])
 {
@@ -575,8 +649,7 @@ sp_item_i2d_affine (SPItem *item, gdouble affine[])
 	g_return_val_if_fail (affine != NULL, NULL);
 
 	sp_item_i2doc_affine (item, affine);
-	art_affine_identity (doc2dt);
-	doc2dt[3] = -1.0;
+	art_affine_scale (doc2dt, 0.8, -0.8);
 	doc2dt[5] = sp_document_height (SP_OBJECT_DOCUMENT (item));
 	art_affine_multiply (affine, affine, doc2dt);
 
@@ -595,8 +668,7 @@ sp_item_set_i2d_affine (SPItem *item, gdouble affine[])
 	if (SP_OBJECT_PARENT (item)) {
 		sp_item_i2d_affine ((SPItem *) SP_OBJECT_PARENT (item), p2d);
 	} else {
-		art_affine_identity (p2d);
-		p2d[3] = -1.0;
+		art_affine_scale (p2d, 0.8, -0.8);
 		p2d[5] = sp_document_height (SP_OBJECT_DOCUMENT (item));
 	}
 
@@ -607,6 +679,18 @@ sp_item_set_i2d_affine (SPItem *item, gdouble affine[])
 	sp_item_set_item_transform (item, i2p);
 }
 
+gdouble *
+sp_item_dt2i_affine (SPItem *item, SPDesktop *dt, gdouble affine[])
+{
+	gdouble i2dt[6];
+
+	/* fixme: Implement the right way (Lauris) */
+	sp_item_i2d_affine (item, i2dt);
+	art_affine_invert (affine, i2dt);
+
+	return affine;
+}
+
 /* Generate context menu item section */
 
 static void
