@@ -17,7 +17,7 @@
 #define SP_IS_NAMEDVIEW(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_NAMEDVIEW))
 #define SP_IS_NAMEDVIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_NAMEDVIEW))
 
-#include <libgnomeui/gnome-canvas.h>
+#include "helper/units.h"
 #include "sp-object-group.h"
 
 struct _SPNamedView {
@@ -27,10 +27,20 @@ struct _SPNamedView {
 	guint snaptogrid : 1;
 	guint showguides : 1;
 	guint snaptoguides : 1;
+
+	const SPUnit *gridunit;
+	/* Grid data is in points regardless of unit */
+	gdouble gridoriginx;
+	gdouble gridoriginy;
+	gdouble gridspacingx;
+	gdouble gridspacingy;
+
+	const SPUnit *gridtoleranceunit;
 	gdouble gridtolerance;
+
+	const SPUnit *guidetoleranceunit;
 	gdouble guidetolerance;
-	ArtPoint gridorigin;
-	ArtPoint gridspacing;
+
 	guint32 gridcolor;
 	guint32 guidecolor;
 	guint32 guidehicolor;
@@ -38,6 +48,7 @@ struct _SPNamedView {
 	GSList * vguides;
 	GSList * views;
 	GSList * gridviews;
+        gint viewcount;
 };
 
 struct _SPNamedViewClass {
@@ -50,6 +61,11 @@ void sp_namedview_show (SPNamedView * namedview, gpointer desktop);
 void sp_namedview_hide (SPNamedView * namedview, gpointer desktop);
 
 void sp_namedview_activate_guides (SPNamedView * nv, gpointer desktop, gboolean active);
+guint sp_namedview_viewcount (SPNamedView * nv);
+const gchar * sp_namedview_get_name (SPNamedView * nv);
+const GSList * sp_namedview_view_list (SPNamedView * nv);
+
+
 
 #endif
 
