@@ -42,6 +42,7 @@ static gchar * sp_object_get_unique_id (SPObject * object, const gchar * defid);
 enum {MODIFIED, LAST_SIGNAL};
 
 SPReprEventVector object_event_vector = {
+	NULL, /* Destroy */
 	NULL, /* Add child */
 	sp_object_repr_child_added,
 	sp_object_repr_remove_child,
@@ -469,22 +470,6 @@ sp_object_repr_content_changed (SPRepr *repr, const guchar *oldcontent, const gu
 	sp_document_content_changed (object->document, object, oldcontent, newcontent);
 }
 
-/* Styling */
-
-/* fixme: this is potentially dangerous - use load/set style instead (Lauris) */
-void
-sp_object_style_changed (SPObject *object, guint flags)
-{
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
-
-	if (((SPObjectClass *)(((GtkObject *) object)->klass))->style_changed)
-		(*((SPObjectClass *)(((GtkObject *) object)->klass))->style_changed) (object, flags);
-
-	/* fixme: I think this is correct - changing parent style modifies object itself */
-	sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-}
-
 /* Modification */
 
 void
@@ -494,17 +479,16 @@ sp_object_request_modified (SPObject *object, guint flags)
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (SP_IS_OBJECT (object));
-	g_return_if_fail ((flags == SP_OBJECT_MODIFIED_FLAG) || (flags == SP_OBJECT_CHILD_MODIFIED_FLAG));
+	g_return_if_fail (!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
+	g_return_if_fail ((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
+	g_return_if_fail (!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
 	/* Check for propagate before we set any flags */
-	propagate = ((GTK_OBJECT_FLAGS (object) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)) == 0);
+	/* Propagate means, that object is not passed through by modification request cascade yet */
+	propagate = (!(SP_OBJECT_FLAGS (object) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-	/* Either one is true, so we can simply overwrite flag */
-	if (flags & SP_OBJECT_MODIFIED_FLAG) {
-		SP_OBJECT_SET_FLAGS (object, SP_OBJECT_MODIFIED_FLAG);
-	} else if (flags & SP_OBJECT_CHILD_MODIFIED_FLAG) {
-		SP_OBJECT_SET_FLAGS (object, SP_OBJECT_CHILD_MODIFIED_FLAG);
-	}
+	/* Just set object flags safe even if some have been set before */
+	SP_OBJECT_SET_FLAGS (object, flags);
 
 	if (propagate) {
 		if (object->parent) {
@@ -520,18 +504,40 @@ sp_object_modified (SPObject *object, guint flags)
 {
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (SP_IS_OBJECT (object));
-	g_return_if_fail ((flags == 0) || (flags == SP_OBJECT_PARENT_MODIFIED_FLAG));
+	g_return_if_fail (!(flags & ~SP_OBJECT_MODIFIED_CASCADE));
 
-	flags |= (GTK_OBJECT_FLAGS (object) & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG));
+	flags |= (GTK_OBJECT_FLAGS (object) & SP_OBJECT_MODIFIED_STATE);
 
 	g_return_if_fail (flags != 0);
 
-	/* Clear flags to allow reentrancy */
-	SP_OBJECT_UNSET_FLAGS (object, (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG));
+	/* Merge style if we have good reasons to think that parent style is changed */
+	/* I am not sure, whether we should check only propagated flag */
+	/* We are currently assuming, that style parsing is done immediately */
+	/* I think this is correct (Lauris) */
+	if (flags & SP_OBJECT_STYLE_MODIFIED_FLAG) {
+		if (object->style && object->parent) {
+			sp_style_merge_from_parent (object->style, object->parent->style);
+		}
+	}
 
 	gtk_object_ref (GTK_OBJECT (object));
 	gtk_signal_emit (GTK_OBJECT (object), object_signals[MODIFIED], flags);
 	gtk_object_unref (GTK_OBJECT (object));
+
+	/* If style is modified, invoke style_modified virtual method */
+	/* It is pure convenience, and should be used with caution */
+	/* The cascade is created solely by modified method plus appropriate flag */
+	/* Also, it merely signals, that actual style object has changed */
+	if (flags & SP_OBJECT_STYLE_MODIFIED_FLAG) {
+		if (((SPObjectClass *)(((GtkObject *) object)->klass))->style_modified)
+			(*((SPObjectClass *)(((GtkObject *) object)->klass))->style_modified) (object, flags);
+	}
+
+	/*
+	 * fixme: I am not sure - it was before class method, but moved it here
+	 */
+
+	SP_OBJECT_UNSET_FLAGS (object, SP_OBJECT_MODIFIED_STATE);
 }
 
 /* Sequence */
@@ -640,6 +646,7 @@ const guchar *
 sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *def)
 {
 	const gchar *style;
+	const guchar *val;
 
 	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
@@ -658,6 +665,8 @@ sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *d
 			if (*p) return p;
 		}
 	}
+	val = sp_repr_attr (object->repr, key);
+	if (val) return val;
 	if (object->parent) {
 		return sp_object_get_style_property (object->parent, key, def);
 	}
