@@ -1,20 +1,54 @@
-#ifndef SP_SVG_H
-#define SP_SVG_H
+#ifndef __SP_SVG_H__
+#define __SP_SVG_H__
+
+/*
+ * SVG data parser
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <glib.h>
 #include <libart_lgpl/art_bpath.h>
 #include <libgnomeprint/gnome-font.h>
+#include "svg-types.h"
 #include "../helper/units.h"
-#include "../style.h"
-#if 0
-#include "../display/fill.h"
-#endif
 #include "../display/stroke.h"
 
-/* General CSS Properties */
-/* length */
+/* Generic */
 
+/*
+ * These are very-very simple:
+ * - they accept everything libc strtod accepts
+ * - no valid end character checking
+ * Return FALSE and let val untouched on error
+ */
+ 
+gboolean sp_svg_read_number_f (const guchar *str, gfloat *val);
+gboolean sp_svg_read_number_d (const guchar *str, gdouble *val);
+
+/* Length */
+
+/*
+ * Parse number with optional unit specifier:
+ * - for px, pt, pc, mm, cm, computed is final value accrding to SVG spec
+ * - for em, ex, and % computed is left untouched
+ * - % is divided by 100 (i.e. 100% is 1.0)
+ * !isalnum check is done at the end
+ * Any return value pointer can be NULL
+ */
+
+gboolean sp_svg_length_read (const unsigned char *str, SPSVGLength *length);
+gboolean sp_svg_length_read_lff (const unsigned char *str, unsigned long *unit, float *value, float *computed);
+void sp_svg_length_unset (SPSVGLength *length, unsigned long unit, float value, float computed);
+
+#if 0
 gdouble sp_svg_read_length (const SPUnit **unit, const gchar *str, gdouble def);
+#endif
 gint sp_svg_write_length (gchar *buf, gint buflen, gdouble val, const SPUnit *unit);
 
 gdouble sp_svg_read_percentage (const gchar * str, gdouble def);
@@ -31,11 +65,6 @@ gint sp_svg_write_affine (gchar * buf, gint buflen, gdouble affine[]);
 ArtBpath * sp_svg_read_path (const gchar * str);
 gchar * sp_svg_write_path (const ArtBpath * bpath);
 
-#if 0
-SPFillType sp_svg_read_fill_type (const gchar * str);
-gint sp_svg_write_fill_type (gchar * buf, gint buflen, SPFillType type, guint32 color);
-#endif
-
 SPStrokeType sp_svg_read_stroke_type (const gchar * str);
 ArtPathStrokeJoinType sp_svg_read_stroke_join (const gchar * str);
 ArtPathStrokeCapType sp_svg_read_stroke_cap (const gchar * str);
