@@ -15,6 +15,7 @@
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
 #include <gal/widgets/e-unicode.h>
+#include <helper/sp-ctrlline.h>
 #include "sp-text.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -27,14 +28,17 @@
 
 static void sp_text_context_class_init (SPTextContextClass * klass);
 static void sp_text_context_init (SPTextContext * text_context);
-static void sp_text_context_destroy (GtkObject * object);
+static void sp_text_context_finalize (GtkObject *object);
 
-static void sp_text_context_setup (SPEventContext * event_context, SPDesktop * desktop);
+static void sp_text_context_setup (SPEventContext *ec);
+static void sp_text_context_finish (SPEventContext *ec);
 static gint sp_text_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_text_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
 
 static void sp_text_context_selection_changed (SPSelection *selection, SPTextContext *tc);
 
+static gint sp_text_context_timeout (SPTextContext *tc);
+
 static SPEventContextClass * parent_class;
 
 GtkType
@@ -72,9 +76,10 @@ sp_text_context_class_init (SPTextContextClass * klass)
 
 	parent_class = gtk_type_class (sp_event_context_get_type ());
 
-	object_class->destroy = sp_text_context_destroy;
+	object_class->finalize = sp_text_context_finalize;
 
 	event_context_class->setup = sp_text_context_setup;
+	event_context_class->finish = sp_text_context_finish;
 	event_context_class->root_handler = sp_text_context_root_handler;
 	event_context_class->item_handler = sp_text_context_item_handler;
 }
@@ -90,13 +95,19 @@ sp_text_context_init (SPTextContext *tc)
 	event_context->hot_x = 0;
 	event_context->hot_y = 0;
 
-	tc->item = NULL;
+	tc->text = NULL;
 	tc->pdoc.x = 0.0;
 	tc->pdoc.y = 0.0;
+	tc->ipos = 0;
+
+	tc->cursor = NULL;
+	tc->timeout = 0;
+	tc->show = FALSE;
+	tc->phase = 0;
 }
 
 static void
-sp_text_context_destroy (GtkObject *object)
+sp_text_context_finalize (GtkObject *object)
 {
 	SPEventContext *ec;
 	SPTextContext *tc;
@@ -104,26 +115,20 @@ sp_text_context_destroy (GtkObject *object)
 	ec = SP_EVENT_CONTEXT (object);
 	tc = SP_TEXT_CONTEXT (object);
 
-#if 0
-	if (text_context->text)
-		sp_text_complete (text_context);
-#endif
+	if (tc->timeout) {
+		gtk_timeout_remove (tc->timeout);
+	}
 
-	gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_CANVAS (ec->desktop)), tc);
-#ifdef SP_TC_XIM
-	gdk_ic_destroy (tc->ic);
-	gdk_ic_attr_destroy (tc->ic_attr);
-	gdk_window_set_events (GTK_WIDGET (SP_DT_CANVAS (ec->desktop))->window, tc->savedmask);
-#endif
-
-	tc->item = NULL;
+	if (tc->cursor) {
+		gtk_object_destroy (GTK_OBJECT (tc->cursor));
+	}
 
 	if (ec->desktop) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_CANVAS (ec->desktop)), tc);
 		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_SELECTION (ec->desktop)), ec);
 	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	GTK_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gint
@@ -141,11 +146,18 @@ sptc_focus_out (GtkWidget *widget, GdkEventFocus *event, SPTextContext *tc)
 }
 
 static void
-sp_text_context_setup (SPEventContext *event_context, SPDesktop *desktop)
+sp_text_context_setup (SPEventContext *ec)
 {
 	SPTextContext *tc;
+	SPDesktop *desktop;
 
-	tc = SP_TEXT_CONTEXT (event_context);
+	tc = SP_TEXT_CONTEXT (ec);
+	desktop = ec->desktop;
+
+	tc->cursor = gnome_canvas_item_new (SP_DT_CONTROLS (desktop), SP_TYPE_CTRLLINE, NULL);
+	sp_ctrlline_set_coords (SP_CTRLLINE (tc->cursor), 100, 0, 100, 100);
+	gnome_canvas_item_hide (tc->cursor);
+	tc->timeout = gtk_timeout_add (500, (GtkFunction) sp_text_context_timeout, ec);
 
 #ifdef SP_TC_XIM
 	if (gdk_im_ready () && (tc->ic_attr = gdk_ic_attr_new ()) != NULL) {
@@ -182,7 +194,7 @@ sp_text_context_setup (SPEventContext *event_context, SPDesktop *desktop)
 	}
 #endif
 	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
-		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (event_context, desktop);
+		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (ec);
 
 	gtk_signal_connect (GTK_OBJECT (SP_DT_SELECTION (desktop)), "changed",
 			    GTK_SIGNAL_FUNC (sp_text_context_selection_changed), tc);
@@ -190,6 +202,35 @@ sp_text_context_setup (SPEventContext *event_context, SPDesktop *desktop)
 	sp_text_context_selection_changed (SP_DT_SELECTION (desktop), tc);
 }
 
+static void
+sp_text_context_finish (SPEventContext *ec)
+{
+	SPTextContext *tc;
+
+	tc = SP_TEXT_CONTEXT (ec);
+
+#ifdef SP_TC_XIM
+	gdk_ic_destroy (tc->ic);
+	gdk_ic_attr_destroy (tc->ic_attr);
+	gdk_window_set_events (GTK_WIDGET (SP_DT_CANVAS (ec->desktop))->window, tc->savedmask);
+#endif
+
+	if (tc->timeout) {
+		gtk_timeout_remove (tc->timeout);
+		tc->timeout = 0;
+	}
+
+	if (tc->cursor) {
+		gtk_object_destroy (GTK_OBJECT (tc->cursor));
+		tc->cursor = 0;
+	}
+
+	if (ec->desktop) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_CANVAS (ec->desktop)), tc);
+		gtk_signal_disconnect_by_data (GTK_OBJECT (SP_DT_SELECTION (ec->desktop)), ec);
+	}
+}
+
 static gint
 sp_text_context_item_handler (SPEventContext *ec, SPItem *item, GdkEvent *event)
 {
@@ -225,8 +266,6 @@ static gint
 sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 {
 	SPTextContext *tc;
-	guchar *utf8;
-	const guchar *content;
 	gint ret;
 
 	tc = SP_TEXT_CONTEXT (ec);
@@ -236,60 +275,78 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+			ArtPoint dtp;
 			/* Button 1, set X & Y & new item */
 			sp_selection_empty (SP_DT_SELECTION (ec->desktop));
 			sp_desktop_w2doc_xy_point (ec->desktop, &tc->pdoc, event->button.x, event->button.y);
+			/* Cursor */
+			tc->show = TRUE;
+			tc->phase = 1;
+			gnome_canvas_item_show (tc->cursor);
+			sp_desktop_w2d_xy_point (ec->desktop, &dtp, event->button.x, event->button.y);
+			sp_ctrlline_set_coords (SP_CTRLLINE (tc->cursor), dtp.x, dtp.y, dtp.x + 32, dtp.y);
+			/* Processed */
 			ret = TRUE;
 		}
 		break;
 	case GDK_KEY_PRESS:
+#if 0
 		/* fixme: */
 		// filter control-modifier for desktop shortcuts
+		/* I removed this, as we prefer some text specific stuff here (Lauris) */
                 if (event->key.state & GDK_CONTROL_MASK) return FALSE;
+#endif
+		if (!tc->text) {
+			SPRepr *rtext, *rtspan, *rstring, *style;
 
-		if (!tc->item) {
-			SPRepr *repr, *style;
-			/* Create object */
-			repr = sp_repr_new ("text");
+			/* Create <text> */
+			rtext = sp_repr_new ("text");
 			/* Set style */
-			style = sodipodi_get_repr (SODIPODI, "paint.text");
+			style = sodipodi_get_repr (SODIPODI, "tools.text");
 			if (style) {
 				SPCSSAttr *css;
 				css = sp_repr_css_attr_inherited (style, "style");
-				sp_repr_css_set (repr, css, "style");
+				sp_repr_css_set (rtext, css, "style");
 				sp_repr_css_attr_unref (css);
 			}
-			sp_repr_set_double_attribute (repr, "x", tc->pdoc.x);
-			sp_repr_set_double_attribute (repr, "y", tc->pdoc.y);
-			tc->item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (ec->desktop), repr);
-			sp_selection_set_item (SP_DT_SELECTION (ec->desktop), tc->item);
+			sp_repr_set_double_attribute (rtext, "x", tc->pdoc.x);
+			sp_repr_set_double_attribute (rtext, "y", tc->pdoc.y);
+
+			/* Create <tspan> */
+			rtspan = sp_repr_new ("tspan");
+			sp_repr_add_child (rtext, rtspan, NULL);
+			sp_repr_unref (rtspan);
+
+			/* Create TEXT */
+			rstring = sp_xml_document_createTextNode (sp_repr_document (rtext), "");
+			sp_repr_add_child (rtspan, rstring, NULL);
+			sp_repr_unref (rstring);
+
+			sp_document_add_repr (SP_DT_DOCUMENT (ec->desktop), rtext);
+			/* fixme: Is selection::changed really immediate? */
+			sp_selection_set_repr (SP_DT_SELECTION (ec->desktop), rtext);
+			sp_repr_unref (rtext);
+
 			sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-			sp_repr_unref (repr);
 		}
-		utf8 = NULL;
+
+		g_assert (tc->text != NULL);
+
 		if (event->key.keyval == GDK_Return) {
-			utf8 = g_strdup ("\n");
+			SPTSpan *new;
+			/* Append new line */
+			/* fixme: Has to be insert here */
+			new = sp_text_append_line (SP_TEXT (tc->text));
+			tc->ipos += 1;
+		} else if ((event->key.keyval == GDK_space) && (event->key.state & GDK_CONTROL_MASK)) {
+			/* Nonbreaking space */
+			tc->ipos = sp_text_append (SP_TEXT (tc->text), "\302\240");
 		} else if (event->key.string) {
-			g_print ("Key %d string %s\n", event->key.keyval, event->key.string);
-#if 0
-			utf8 = e_utf8_from_gtk_string (GTK_WIDGET (ec->desktop->owner->canvas), event->key.string);
-#else
+			guchar *utf8;
 			utf8 = e_utf8_from_locale_string (event->key.string);
-#endif
-			g_print ("UTF8 %s\n", utf8);
+			tc->ipos = sp_text_append (SP_TEXT (tc->text), utf8);
+			if (utf8) g_free (utf8);
 		}
-		content = sp_repr_content (SP_OBJECT_REPR (tc->item));
-		if (content) {
-			guchar *new;
-			new = g_strconcat (content, utf8, NULL);
-			g_print ("Old %s new %s\n", content, new);
-			sp_repr_set_content (SP_OBJECT_REPR (tc->item), new);
-			g_free (new);
-		} else {
-			g_print ("Setting %s\n", utf8);
-			sp_repr_set_content (SP_OBJECT_REPR (tc->item), utf8);
-		}
-		if (utf8) g_free (utf8);
 		sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 
 		ret = TRUE;
@@ -306,45 +363,40 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	return ret;
 }
 
-#if 0
-static void
-sp_text_complete (SPTextContext * text_context)
-{
-	gchar * c;
-
-	if (text_context->text == NULL) return;
-
-	c = (gchar *) sp_repr_content (text_context->text);
-	if (c != NULL) {
-		if (*c != '\0') {
-			gdk_keyboard_ungrab (gdk_time_get ());
-			text_context->canvasitem = NULL;
-			text_context->item = NULL;
-			text_context->text = NULL;
-			return;
-		}
-	}
-	g_print ("empty text created - removing\n");
-	sp_repr_unparent (text_context->text);
-
-	sp_document_done (SP_DT_DOCUMENT (SP_EVENT_CONTEXT (text_context)->desktop));
-
-	text_context->text = NULL;
-}
-#endif
-
 static void
 sp_text_context_selection_changed (SPSelection *selection, SPTextContext *tc)
 {
 	SPItem *item;
 
-	tc->item = NULL;
+	tc->text = NULL;
 
 	item = sp_selection_item (selection);
 
 	if (SP_IS_TEXT (item)) {
-		tc->item = item;
+		tc->text = item;
+		tc->ipos = sp_text_get_length (SP_TEXT (tc->text));
+		gnome_canvas_item_show (tc->cursor);
+		tc->show = TRUE;
+		tc->phase = 1;
+	} else {
+		tc->text = NULL;
+		gnome_canvas_item_hide (tc->cursor);
+		tc->show = FALSE;
 	}
 }
 
+static gint
+sp_text_context_timeout (SPTextContext *tc)
+{
+	if (tc->show) {
+		if (tc->phase) {
+			tc->phase = 0;
+			gnome_canvas_item_hide (tc->cursor);
+		} else {
+			tc->phase = 1;
+			gnome_canvas_item_show (tc->cursor);
+		}
+	}
 
+	return TRUE;
+}
