@@ -29,8 +29,17 @@ typedef struct _SPTextContextClass SPTextContextClass;
 struct _SPTextContext {
 	SPEventContext event_context;
 
-	SPItem *item;
+	SPItem *text;
+
+	/* Text item position in document coordinates */
 	ArtPoint pdoc;
+	/* Insertion point position */
+	gint ipos;
+
+	GnomeCanvasItem *cursor;
+	gint timeout;
+	guint show : 1;
+	guint phase : 1;
 
 #ifdef SP_TC_XIM
 	GdkIC *ic;
