@@ -1,20 +1,34 @@
-#define SP_REPR_C
+#define __SP_REPR_C__
+
+/*
+ * Fuzzy DOM-like tree implementation
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 1999-2002 authors
+ * Copyright (C) 2000-2002 Ximian, Inc.
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
 #include <string.h>
 #include "repr-private.h"
 
+typedef struct _SPReprListener SPListener;
+
 static SPRepr * sp_repr_new_from_code (gint code);
 static void sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr);
-static void sp_node_remove_listener (SPNode *node, SPListener *listener);
+static void sp_node_remove_listener (SPXMLNode *node, SPListener *listener);
 
-static SPAttribute *sp_attribute_duplicate (const SPAttribute *attr);
-static SPAttribute *sp_attribute_new_from_quark (gint key, const guchar *value);
+static SPXMLAttribute *sp_attribute_duplicate (const SPXMLAttribute *attr);
+static SPXMLAttribute *sp_attribute_new_from_quark (gint key, const guchar *value);
 
 /* Helpers */
-static SPNode * sp_node_alloc (void);
-static void sp_node_free (SPNode *node);
-static SPAttribute * sp_attribute_alloc (void);
-static void sp_attribute_free (SPAttribute *attribute);
+static SPXMLNode * sp_node_alloc (void);
+static void sp_node_free (SPXMLNode *node);
+static SPXMLAttribute * sp_attribute_alloc (void);
+static void sp_attribute_free (SPXMLAttribute *attribute);
 static SPListener *sp_listener_alloc (void);
 static void sp_listener_free (SPListener *listener);
 
@@ -27,6 +41,7 @@ sp_repr_new_from_code (gint code)
 
 	repr->refcount = 1;
 	repr->name = code;
+	repr->type = SP_XML_ELEMENT_NODE;
 	repr->parent = repr->next = repr->children = NULL;
 	repr->attributes = NULL;
 	repr->listeners = NULL;
@@ -64,9 +79,13 @@ sp_repr_unref (SPRepr *repr)
 	repr->refcount -= 1;
 
 	if (repr->refcount < 1) {
+		SPReprListener *rl;
 		/* Parents reference children */
 		g_assert (repr->parent == NULL);
 		g_assert (repr->next == NULL);
+		for (rl = repr->listeners; rl; rl = rl->next) {
+			if (rl->vector->destroy) (* rl->vector->destroy) (repr, rl->data);
+		}
 		while (repr->children) sp_repr_remove_child (repr, repr->children);
 		while (repr->attributes) sp_repr_remove_attribute (repr, repr->attributes);
 		if (repr->content) g_free (repr->content);
@@ -99,6 +118,7 @@ sp_repr_duplicate (const SPRepr *repr)
 	g_return_val_if_fail (repr != NULL, NULL);
 
 	new = sp_repr_new_from_code (repr->name);
+	new->type = repr->type;
 
 	if (repr->content != NULL) new->content = g_strdup (repr->content);
 
@@ -484,7 +504,7 @@ sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, gpointer da
 }
 
 static void
-sp_node_remove_listener (SPNode *node, SPListener *listener)
+sp_node_remove_listener (SPXMLNode *node, SPListener *listener)
 {
 	if (listener == node->listeners) {
 		node->listeners = listener->next;
@@ -515,7 +535,6 @@ sp_repr_remove_listener_by_data (SPRepr * repr, gpointer data)
 			return;
 		}
 		last = rl;
-		rl = rl->next;
 	}
 }
 
@@ -586,7 +605,7 @@ sp_repr_document_set_root (SPReprDoc * doc, SPRepr * repr)
 }
 
 SPReprDoc *
-sp_repr_document (SPRepr * repr)
+sp_repr_document (const SPRepr *repr)
 {
 	/* fixme: */
 	while (repr->parent) repr = repr->parent;
@@ -595,7 +614,7 @@ sp_repr_document (SPRepr * repr)
 }
 
 SPRepr *
-sp_repr_document_root (SPReprDoc * doc)
+sp_repr_document_root (const SPReprDoc *doc)
 {
 	SPRepr * repr;
 
@@ -605,87 +624,74 @@ sp_repr_document_root (SPReprDoc * doc)
 	return repr->children;
 }
 
-/**
- * sp_repr_document_overwrite:
- * @doc: overwrite into
- * @source: overwrite it
- * @key: identify each node by
- *
- * Copy all @source subtree into @doc.
+/*
+ * Duplicate all attributes and children from src into doc
+ * Does NOT erase original attributes and children
  */
-void
-sp_repr_document_overwrite (SPReprDoc       *doc,
-			    const SPReprDoc *source,
-			    const guchar    *key)
+
+gboolean
+sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const guchar *key)
 {
-	SPRepr       *rdoc;
+	SPRepr *rdoc;
 	const SPRepr *rsrc;
 	
+	g_return_val_if_fail (doc != NULL, FALSE);
+	g_return_val_if_fail (src != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+
 	rdoc = sp_repr_document_root (doc);
-	rsrc = sp_repr_document_root ((SPReprDoc *) source);
+	rsrc = sp_repr_document_root (src);
 	
-	sp_repr_overwrite (rdoc->children, rsrc->children, key);
+	return sp_repr_merge (rdoc, rsrc, key);
 }
 
-/**
- * sp_repr_overwrite:
- * @repr: overwrite into
- * @src: overwrite it
- * @key: identify each node by
- *
- * Copy all @src subtree into @doc.
+/*
+ * Duplicate all attributes and children from src into doc
+ * Does NOT erase original attributes and children
  */
+
 gboolean
-sp_repr_overwrite (SPRepr       *repr,
-		   const SPRepr *src,
-		   const guchar *key)
+sp_repr_merge (SPRepr *repr, const SPRepr *src, const guchar *key)
 {
 	SPRepr *child;
-	SPReprAttr *attr, *lastattr;
+	SPReprAttr *attr;
 	
 	g_return_val_if_fail (repr != NULL, FALSE);
 	g_return_val_if_fail (src != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
 
-	/* Fixme: we should use hash table for faster lookup? */
-	
-	if (src->content != NULL) {
-		if (repr->content != NULL) g_free (repr->content);
+	if (src->content) {
+		if (repr->content) g_free (repr->content);
 		repr->content = g_strdup (src->content);
+	} else {
+		if (repr->content) g_free (repr->content);
+		repr->content = NULL;
 	}
 	
 	for (child = src->children; child != NULL; child = child->next) {
-		SPRepr *rchild;
-		const guchar *value;
-		
-		value = sp_repr_attr (child, key);
-		rchild = sp_repr_lookup_child (repr, key, value);
-		if (rchild) {
-			sp_repr_overwrite (rchild, child, key);
+		SPRepr *rch;
+		const guchar *id;
+		id = sp_repr_attr (child, key);
+		rch = sp_repr_lookup_child (repr, key, id);
+		if (rch) {
+			sp_repr_merge (rch, child, key);
 		} else {
-			rchild = sp_repr_duplicate (child);
-			sp_repr_append_child (repr, rchild);
+			rch = sp_repr_duplicate (child);
+			sp_repr_append_child (repr, rch);
 		}
 	}
 	
-	/* fixme: We probably should merge, not replace here (Lauris) */
-	while (repr->attributes) sp_repr_remove_attribute (repr, repr->attributes);
-
-	lastattr = NULL;
 	for (attr = src->attributes; attr != NULL; attr = attr->next) {
-		if (lastattr) {
-			lastattr = lastattr->next = sp_attribute_duplicate (attr);
-		} else {
-			lastattr = repr->attributes = sp_attribute_duplicate (attr);
-		}
+		sp_repr_set_attr (repr, SP_REPR_ATTRIBUTE_KEY (attr), SP_REPR_ATTRIBUTE_VALUE (attr));
 	}
 
 	return TRUE;
 }
 
-static SPAttribute *
-sp_attribute_duplicate (const SPAttribute *attr)
+static SPXMLAttribute *
+sp_attribute_duplicate (const SPXMLAttribute *attr)
 {
-	SPAttribute *new;
+	SPXMLAttribute *new;
 
 	new = sp_attribute_alloc ();
 
@@ -696,10 +702,10 @@ sp_attribute_duplicate (const SPAttribute *attr)
 	return new;
 }
 
-static SPAttribute *
+static SPXMLAttribute *
 sp_attribute_new_from_quark (gint key, const guchar *value)
 {
-	SPAttribute *new;
+	SPXMLAttribute *new;
 
 	new = sp_attribute_alloc ();
 
@@ -712,19 +718,19 @@ sp_attribute_new_from_quark (gint key, const guchar *value)
 
 /* Helpers */
 #define SP_NODE_ALLOC_SIZE 32
-static SPNode *free_node = NULL;
+static SPXMLNode *free_node = NULL;
 
-static SPNode *
+static SPXMLNode *
 sp_node_alloc (void)
 {
-	SPNode *node;
+	SPXMLNode *node;
 	gint i;
 
 	if (free_node) {
 		node = free_node;
 		free_node = free_node->next;
 	} else {
-		node = g_new (SPNode, SP_NODE_ALLOC_SIZE);
+		node = g_new (SPXMLNode, SP_NODE_ALLOC_SIZE);
 		for (i = 1; i < SP_NODE_ALLOC_SIZE - 1; i++) node[i].next = node + i + 1;
 		node[SP_NODE_ALLOC_SIZE - 1].next = NULL;
 		free_node = node + 1;
@@ -734,26 +740,26 @@ sp_node_alloc (void)
 }
 
 static void
-sp_node_free (SPNode *node)
+sp_node_free (SPXMLNode *node)
 {
 	node->next = free_node;
 	free_node = node;
 }
 
 #define SP_ATTRIBUTE_ALLOC_SIZE 32
-static SPAttribute *free_attribute = NULL;
+static SPXMLAttribute *free_attribute = NULL;
 
-static SPAttribute *
+static SPXMLAttribute *
 sp_attribute_alloc (void)
 {
-	SPAttribute *attribute;
+	SPXMLAttribute *attribute;
 	gint i;
 
 	if (free_attribute) {
 		attribute = free_attribute;
 		free_attribute = free_attribute->next;
 	} else {
-		attribute = g_new (SPAttribute, SP_ATTRIBUTE_ALLOC_SIZE);
+		attribute = g_new (SPXMLAttribute, SP_ATTRIBUTE_ALLOC_SIZE);
 		for (i = 1; i < SP_ATTRIBUTE_ALLOC_SIZE - 1; i++) attribute[i].next = attribute + i + 1;
 		attribute[SP_ATTRIBUTE_ALLOC_SIZE - 1].next = NULL;
 		free_attribute = attribute + 1;
@@ -763,7 +769,7 @@ sp_attribute_alloc (void)
 }
 
 static void
-sp_attribute_free (SPAttribute *attribute)
+sp_attribute_free (SPXMLAttribute *attribute)
 {
 	attribute->next = free_attribute;
 	free_attribute = attribute;
