@@ -15,7 +15,9 @@ static void sp_zoom_context_class_init (SPZoomContextClass * klass);
 static void sp_zoom_context_init (SPZoomContext * zoom_context);
 static void sp_zoom_context_destroy (GtkObject * object);
 
+#if 0
 static void sp_zoom_context_setup (SPEventContext * event_context, SPDesktop * desktop);
+#endif
 static gint sp_zoom_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_zoom_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
 
@@ -63,7 +65,9 @@ sp_zoom_context_class_init (SPZoomContextClass * klass)
 
 	object_class->destroy = sp_zoom_context_destroy;
 
+#if 0
 	event_context_class->setup = sp_zoom_context_setup;
+#endif
 	event_context_class->root_handler = sp_zoom_context_root_handler;
 	event_context_class->item_handler = sp_zoom_context_item_handler;
 }
@@ -87,12 +91,14 @@ sp_zoom_context_destroy (GtkObject * object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+#if 0
 static void
 sp_zoom_context_setup (SPEventContext * event_context, SPDesktop * desktop)
 {
 	if (SP_EVENT_CONTEXT_CLASS (parent_class)->setup)
 		SP_EVENT_CONTEXT_CLASS (parent_class)->setup (event_context, desktop);
 }
+#endif
 
 static gint
 sp_zoom_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
@@ -214,7 +220,7 @@ sp_zoom_drawing (GtkWidget * widget)
 
 	g_return_if_fail (docitem != NULL);
 
-	sp_item_bbox (docitem, &d);
+	sp_item_bbox_desktop (docitem, &d);
 	if ((fabs (d.x1 - d.x0) < 1.0) || (fabs (d.y1 - d.y0) < 1.0)) return;
 	sp_desktop_show_region (desktop, d.x0, d.y0, d.x1, d.y1, 10);
 }
